import matplotlib.pyplot as plt
from pyqchem.plots import plot_configuration
from pyqchem.structure import Structure
from urllib.request import urlopen
import requests as req
import numpy as np
import json
import warnings
from requests.exceptions import ConnectionError


def print_excited_states(parsed_data, include_conf_rasci=False, include_mulliken_rasci=False):
    for i, state in enumerate(parsed_data):
        print('\nState {}'.format(i+1))
        if 'multiplicity' in state:
            print('Multiplicity', state['multiplicity'])

        if state['transition_moment'] is not None:
            #print('Osc. strength: {:6.4f}'.format(state['oscillator_strength']))
            print('Transition DM: ', '{:6.4f} {:6.4f} {:6.4f}'.format(*state['transition_moment']),
                  ' ', state['dipole_moment_units'])

        print('Energy: ', state['excitation_energy'], ' ', state['excitation_energy_units'])

        if include_conf_rasci:
            print('    Configurations')
            print('  Hole   Alpha  Beta   Part   Amplitude')
            for j, conf in enumerate(state['configurations']):
                print(' {:^6} {:^6} {:^6} {:^6} {:8.3f}'.format(conf['hole'], conf['alpha'], conf['beta'], conf['part'], conf['amplitude']))

        if include_mulliken_rasci:
            print('Mulliken analysis')
            print('         Attach    Detach    Total ')
            for i_atom, (at, det) in enumerate(zip(state['mulliken']['attach'],
                                                   state['mulliken']['detach'])):
                print('{:5}  {:8.4f}  {:8.4f}  {:8.4f}'.format(i_atom + 1, at, det, at + det))


def plot_rasci_state_configurations(states):
    for i, state in enumerate(states):
        plt.figure(figsize=(len(state['configurations']), 5))
        plt.title('State {}'.format(i+1))
        amplitude_list = []
        for j, conf in enumerate(state['configurations']):
            plot_configuration(conf['alpha'], conf['beta'], index=j)
            amplitude_list.append(conf['amplitude'])

        plt.plot(range(1, len(amplitude_list)+1), np.square(amplitude_list)*len(state['configurations'][0]['alpha']), label='amplitudes')
        plt.xlabel('Configurations')
        plt.ylabel('Amplitude')
        plt.axis('off')
        plt.legend()

    plt.show()


def submit_notice(message, service='pushbullet', pb_token=None, sp_url=None):
    """
    Submit a notification using webhooks

    :param message: The message to send
    :param service: pushbullet or samepage
    :param pb_token: pushbullet token
    :param sp_url: samepage url
    :return:
    """

    if service.lower() == 'pushbullet':
        url = 'https://api.pushbullet.com/v2/pushes'
        bot_message = {'body': message, 'type': 'note'}
        message_headers = {'Content-Type': 'application/json; charset=UTF-8',
                           'Access-Token': pb_token}
    elif service.lower() == 'samepage':
        url = sp_url
        bot_message = {'text': message}
        message_headers = {'Content-Type': 'application/json'}
    else:
        print('client not found!')
        return

    try:
        r = req.post(url=url,
                     headers=message_headers,
                     data=json.dumps(bot_message))
        r.close()

    except ConnectionError:
        warnings.warn('Connection error: Message was not delivered')


def rotate_coordinates(coordinates, angle, axis, atoms_list=None, center=(0, 0, 0)):
    """
    Rotate the coordinates (or range of coordinates) with respect a given axis

    :param coordinates: coordinates to rotate
    :param angle: rotation angle
    :param axis: rotation axis
    :param atoms_list: list of atoms to rotate (if None then rotate all)
    :return: rotated coordinates
    """
    coordinates = np.array(coordinates) - np.array(center)

    cos_term = 1 - np.cos(angle)
    rot_matrix = [[axis[0]**2*cos_term + np.cos(angle), axis[0]*axis[1]*cos_term - axis[2]*np.sin(angle), axis[0]*axis[2]*cos_term + axis[1]*np.sin(angle)],
                  [axis[1]*axis[0]*cos_term + axis[2]*np.sin(angle), axis[1]**2*cos_term + np.cos(angle), axis[1]*axis[2]*cos_term + axis[0]*np.sin(angle)],
                  [axis[2]*axis[0]*cos_term + axis[1]*np.sin(angle), axis[1]*axis[2]*cos_term + axis[0]*np.sin(angle), axis[2]**2*cos_term + np.cos(angle)]]

    if atoms_list is not None:
        coordinates[atoms_list] = np.dot(coordinates[atoms_list], rot_matrix)
    else:
        coordinates = np.dot(coordinates, rot_matrix) + np.array(center)

    return coordinates.tolist()


def get_geometry_from_pubchem(entry, type='name'):
    """
    Get structure form PubChem database

    :param entry: entry data
    :param type: data type: 'name', 'cid'
    :return: Structure
    """

    base = "https://pubchem.ncbi.nlm.nih.gov/rest/pug/"
    input_1 = 'compound/{}/'.format(type)
    output_format = "JSON"
    additional = "record_type=3d"

    apiurl = base + input_1 + output_format + '?' + additional
    postdata = '{}={}'.format(type, entry).encode()

    from urllib.error import HTTPError

    try:
        response = urlopen(apiurl, postdata)
    except HTTPError as e:
        string = e.read().decode("utf-8")
        json_data = json.loads(string)
        fault = json_data['Fault']
        if 'Details' in fault:
            raise Exception(fault['Details'][0])
        else:
            raise Exception(fault['Message'])

    string = response.read().decode("utf-8")
    json_data = json.loads(string)

    conformers = json_data['PC_Compounds'][0]['coords'][0]['conformers'][0]
    atoms = json_data['PC_Compounds'][0]['atoms']

    positions = np.array([conformers['x'], conformers['y'], conformers['z']]).T
    atomic_numbers = atoms['element']

    if 'charge' in atoms:
        charge = np.add.reduce([c_atom['value'] for c_atom in atoms['charge']])
    else:
        charge = 0

    return Structure(coordinates=positions,
                     atomic_numbers=atomic_numbers,
                     charge=charge)


if __name__ == '__main__':

    mol = get_geometry_from_pubchem('acetone', type='name')
    print(mol)



import numpy as np

from pyqchem.symmetry import get_wf_symmetry
from pyqchem.utils import get_plane, _set_zero_to_coefficients
from pyqchem.qchem_core import get_output_from_qchem
from pyqchem.qc_input import QchemInput
from pyqchem.structure import Structure
from pyqchem.file_io import build_fchk
from copy import deepcopy
from pyqchem.parsers.parser_rasci import parser_rasci
from pyqchem.utils import reorder_coefficients
from pyqchem.symmetry import get_state_symmetry


def inverse_cummulative(vector):
    vector = np.array(vector)

    for i in range(1, len(vector)):
        vector[-(i+1)] -= np.sum(vector[-i:])

    return vector


sym_matrix = {'Ag': np.diag([1, 1]),
              'Au': np.diag([1, -1]),
              'Bg': np.diag([-1,  1]),
              'Bu': np.diag([-1, -1])}


def get_location(occupancies, coefficients, overlap_matrix, error=0.2):
    # For singlets only
    alpha_list = []
    beta_list = []
    mul = 1
    i = 0
    for orbital in coefficients['alpha']:
        occ = occupancies['alpha'][i]
        dot = np.dot(np.dot(orbital, overlap_matrix), orbital)
        print('dot->',i+1,  dot)
        if dot < error:
            alpha_list.append(0)
            beta_list.append(0)
        else:
            if occ < error:
                # print('0')
                alpha_list.append(0)
                beta_list.append(0)
            elif 1+error > occ > error:
                # print('1')
                if mul == 1:
                    alpha_list.append(1)
                    beta_list.append(0)

                    mul += 1
                else:
                    alpha_list.append(0)
                    beta_list.append(1)
                    mul -= 1
            elif occ > 2-error:
                # print('2')
                alpha_list.append(1)
                beta_list.append(1)
            i +=1

    print(alpha_list)
    print(beta_list)

    return {'alpha': alpha_list, 'beta': beta_list}


def orbital_localization2(coefficients, overlap_matrix):
    n = 1
    for a_orbital, b_orbital in zip(coefficients['alpha'], coefficients['beta']):
        a_dot = np.dot(np.dot(a_orbital, overlap_matrix), a_orbital)
        b_dot = np.dot(np.dot(b_orbital, overlap_matrix), b_orbital)

        print('dot', n, a_dot, b_dot)
        n += 1
    return

def get_state_classification(structure,
                             basis,
                             nato_coeff,
                             nato_occup,
                             center,
                             orientation,
                             orbitals,
                             overlap_matrix):

    occupations = get_location(nato_occup, nato_coeff, overlap_matrix)

    reordered_coefficients = reorder_coefficients(occupations, nato_coeff)

    orbital_localization2(reordered_coefficients,overlap_matrix)

    molsym = get_wf_symmetry(structure,
                             basis,
                             reordered_coefficients,
                             center=center,
                             orientation=orientation,
                             group='D2h')

    # molsym.print_alpha_mo_IRD()
    # molsym.print_beta_mo_IRD()
    molsym.print_wf_mo_IRD()
    print('  wf' + '  '.join(['{:7.3f}'.format(s) for s in molsym.wf_IRd]))

    print('\nAlpha MOs: Irred. Rep. Decomposition')
    print('     ' + '  '.join(['{:^7}'.format(s) for s in molsym.IRLab]))
    for i, line in zip(orbitals, molsym.mo_IRd_a[orbitals]):
        print('{:4d}'.format(i + 1) + '  '.join(['{:7.3f}'.format(s) for s in line]))

    print('\nBeta MOs: Irred. Rep. Decomposition')
    print('     ' + '  '.join(['{:^7}'.format(s) for s in molsym.IRLab]))
    for i, line in zip(orbitals, molsym.mo_IRd_b[orbitals]):
        print('{:4d}'.format(i + 1) + '  '.join(['{:7.3f}'.format(s) for s in line]))

    print('occupations')
    print('alpha', np.array(nato_occup['alpha'])[orbitals])
    print('beta ', np.array(nato_occup['beta'])[orbitals])








ethene = [[0.0,  0.0000,   0.65750],
          [0.0,  0.0000,  -0.65750],
          [0.0,  0.92281,  1.22792],
          [0.0, -0.92281,  1.22792],
          [0.0, -0.92281, -1.22792],
          [0.0,  0.92281, -1.22792]]

symbols = ['C', 'C', 'H', 'H', 'H', 'H']


# create molecule
molecule = Structure(coordinates=ethene,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)


print('Optimized monomer structure')
print(molecule)

# Build dimer from monomer
coor_monomer2 = np.array(ethene)

from kimonet.utils.rotation import rotate_vector
coor_monomer2 = np.array([rotate_vector(coor, [0.0, 0.0, 0.0]) for coor in coor_monomer2])


coor_monomer2[:, 0] += 4.0  # monomers separation
coor_monomer2[:, 1] += 0.0  # monomers separation
coor_monomer2[:, 2] += 0.0  # monomers separation

coordinates = ethene + coor_monomer2.tolist()
symbols_dimer = symbols * 2

dimer = Structure(coordinates=coordinates,
                  symbols=symbols_dimer,
                  charge=0,
                  multiplicity=1)

print(dimer.get_xyz())

diabatization_scheme = [
                        # {'method': 'ER', 'states': [1, 2, 3, 4]},  # in order with respect to selected states
                        {'method': 'DQ', 'states': [1, 2, 3, 4], 'parameters': 0.0}
                        ]

# create Q-Chem input
qc_input = QchemInput(dimer,
                      jobtype='sp',
                      exchange='hf',
                      correlation='rasci',
                      ras_act=6,
                      ras_elec=4,
                      basis='sto-3g',
                      ras_spin_mult=1,  # singlets only
                      ras_roots=8,
                      ras_do_hole=False,
                      ras_do_part=False,
                      ras_diabatization_states=[3, 4, 5, 6],  # adiabatic states selected for diabatization
                      ras_diabatization_scheme=diabatization_scheme,
                      )

print(qc_input.get_txt())


# get data from Q-Chem calculation
output, electronic_structure = get_output_from_qchem(qc_input,
                                                          processors=4,
                                                          force_recalculation=False,
                                                          read_fchk=True,
                                                          parser=parser_rasci,
                                                          store_full_output=True)


def is_equal(conf1, conf2):
    lv = []
    for prop in ['alpha', 'beta', 'hole', 'part']:
        lv.append(conf1[prop] == conf2[prop])
    return np.all(lv)


def unify_configurations(configurations_list):
    new_configurations= []

    for conf in configurations_list:
        skip = False
        for n_conf in new_configurations:
            if is_equal(n_conf, conf):
                n_conf['amplitude'] += conf['amplitude']
                skip = True
        if not skip:
            new_configurations.append(conf)

    return new_configurations


print(output)
print(output['excited_states'])


print('++++++++')

print(len(electronic_structure['nato_coefficients_multi']))




# Just for testing
# electronic_structure['coefficients'] = electronic_structure['nato_coefficients']
# electronic_structure['mo_energies'] = electronic_structure['nato_occupancies']

print(len(electronic_structure['nato_coefficients_multi']))
print(len(electronic_structure['nato_occupancies_multi']))
print(electronic_structure['nato_occupancies_multi'])

# store original fchk info in file
es = deepcopy(electronic_structure)
es['coefficients'] = electronic_structure['nato_coefficients_multi'][1]
es['mo_energies'] = electronic_structure['nato_occupancies_multi'][1]

open('test.fchk', 'w').write(build_fchk(es))


# print(electronic_structure['nato_coefficients'])

range_f1 = range(0, 6)
range_f2 = range(6, 12)



mo_coeff_f1 = _set_zero_to_coefficients(electronic_structure['basis'],
                                        electronic_structure['nato_coefficients_multi'][1],
                                        range_f2)



# get symmetry classification
electronic_structure['coefficients'] = mo_coeff_f1
electronic_structure['mo_energies'] = electronic_structure['nato_occupancies']

# save test fchk file with new coefficients
open('test_f1.fchk', 'w').write(build_fchk(electronic_structure))


# get plane from coordinates
coordinates_f1 = np.array(electronic_structure['structure'].get_coordinates())[range_f1]
center_f1, normal_f1 = get_plane(coordinates_f1)


mulliken = output['diabatization']['mulliken_analysis'][1]
a = np.sum(np.array(mulliken['attach'])[range_f1])
print(a)
print(electronic_structure['overlap'])

print(mulliken)

def orbital_localization(coefficients, overlap_matrix):
    for a_orbital, b_orbital in zip(coefficients['alpha'], coefficients['beta']):
        a_dot = np.dot(np.dot(a_orbital, overlap_matrix), a_orbital)
        b_dot = np.dot(np.dot(b_orbital, overlap_matrix), b_orbital)

        print('dot', a_dot, b_dot)
    return

# orbital_localization(mo_coeff_f1, electronic_structure['overlap'])


# get classified orbitals
get_state_classification(electronic_structure['structure'],
                         electronic_structure['basis'],
                         mo_coeff_f1,
                         electronic_structure['nato_occupancies_multi'][1],
                         center=center_f1,
                         orientation=normal_f1,
                         orbitals=[10, 11, 12, 13, 14, 15],
                         overlap_matrix=electronic_structure['overlap'])




import numpy as np
import re
from pyqchem.utils import get_occupied_electrons


def read_symmetry_info(output):
    enum = output.find('Molecular Point Group')
    mpg = output[enum:enum + 100].split()[3]
    enum = output.find('Largest Abelian Subgroup')
    if enum > 0:
        las = output[enum:enum + 100].split()[3]
    else:
        las = None

    return {'molecular_point_group': mpg,
            'largest_abelian_subgroup': las}


def read_basic_info(output):

    there_vector = [m.start() for m in re.finditer('There are ', output)]
    n_alpha = int(output[there_vector[0]:there_vector[0]+100].split()[2])
    n_beta = int(output[there_vector[0]:there_vector[0]+100].split()[5])

    nshell = int(output[there_vector[1]:there_vector[1]+100].split()[2])
    nbas = int(output[there_vector[1]:there_vector[1]+100].split()[5])

    return {'n_alpha': n_alpha,
            'n_beta': n_beta,
            'n_shells': nshell,
            'n_basis_functions': nbas}


def get_cis_occupations_list(number_of_orbitals,
                             alpha_electrons,
                             beta_electrons,
                             alpha_transitions=(),
                             beta_transitions=(),
                             ground_state=None):

    if ground_state is None:
        alpha_occupation = [1] * alpha_electrons + (number_of_orbitals - alpha_electrons) * [0]
        beta_occupation = [1] * beta_electrons + (number_of_orbitals - beta_electrons) * [0]
    else:
        alpha_occupation = ground_state['alpha']
        beta_occupation = ground_state['beta']

    for transition in alpha_transitions:
        alpha_occupation[transition['origin'] - 1] = 0
        alpha_occupation[transition['target'] - 1] = 1

    for transition in beta_transitions:
        beta_occupation[transition['origin'] - 1] = 0
        beta_occupation[transition['target'] - 1] = 1

    return {'alpha': alpha_occupation,
            'beta': beta_occupation}


def get_rasci_occupations_list(configuration, structure, total_orbitals):
    occupied_orbitals = get_occupied_electrons(configuration, structure)
    n_extra = total_orbitals - occupied_orbitals - len(configuration['alpha'])
    vector_alpha = [1] * occupied_orbitals + [int(c) for c in configuration['alpha']] + [0] * n_extra

    n_extra = total_orbitals - occupied_orbitals - len(configuration['beta'])
    vector_beta = [1] * occupied_orbitals + [int(c) for c in configuration['beta']] + [0] * n_extra

    if configuration['hole'] != '':
        if np.sum(vector_alpha) > np.sum(vector_beta):
            vector_alpha[int(configuration['hole']) - 1] = 0
        else:
            vector_beta[int(configuration['hole']) - 1] = 0

    if configuration['part'] != '':
        if np.sum(vector_alpha) < np.sum(vector_beta):
            vector_alpha[int(configuration['part']) - 1] = 1
        else:
            vector_beta[int(configuration['part']) - 1] = 1

    return {'alpha': vector_alpha, 'beta': vector_beta}


def search_bars(output, from_position=0, bar_type='---'):
    output = output[from_position:]
    positions = []
    previous = 0
    for m in re.finditer(bar_type, output):
        if m.start() > previous + 1:
            positions.append(m.start() + from_position)
        previous = m.end()

    return positions


def standardize_vector(vector):
    import numpy as np
    if vector[0] != 0:
        if vector[0] < 0:
            vector = np.array(vector) * -1
            vector = vector.tolist()
    elif vector[1] != 0:
        if vector[1] < 0:
            vector = np.array(vector) * -1
            vector = vector.tolist()
    else:
        if vector[2] < 0:
            vector = np.array(vector) * -1
            vector = vector.tolist()

    for i in range(3):
        vector[i] = vector[i] + 0

    return vector




import numpy as np


def unit_vector(vector):
    """
    Compute unit vector from general vector

    :param vector: the vector
    :return: the unit vector
    """
    return np.array(vector) / np.linalg.norm(vector)


def get_dihedral(coordinates, atoms):
    """
    Compute the dihedral angle between 4 atoms

    :param coordinates: list of coordinates of the molecule
    :param atoms: list of 4 atom indices to use to calculate the dihedral angle (from 1 to N)
    :return: the dihedral angle
    """

    b1 = np.array(coordinates[atoms[1]-1]) - np.array(coordinates[atoms[0]-1])
    b2 = np.array(coordinates[atoms[2]-1]) - np.array(coordinates[atoms[1]-1])
    b3 = np.array(coordinates[atoms[3]-1]) - np.array(coordinates[atoms[2]-1])

    n1 = unit_vector(np.cross(b1, b2))
    n2 = unit_vector(np.cross(b2, b3))

    b2 = unit_vector(b2)
    m = np.cross(n1, b2)

    assert np.abs(np.dot(b2, n2)) < 1e-8

    # sign given by convention https://en.wikipedia.org/wiki/Dihedral_angle
    return -np.rad2deg(np.arctan2(np.dot(m, n2), np.dot(n1, n2)))


def get_angle(coordinates, atoms):
    """
    Compute the angle between 3 atoms

    :param coordinates: list of coordinates of the molecule
    :param atoms: list of 3 atom indices to use to calculate the angle (from 1 to N)
    :return: the angle
    """

    b1 = np.array(coordinates[atoms[0]-1]) - np.array(coordinates[atoms[1]-1])
    b2 = np.array(coordinates[atoms[2]-1]) - np.array(coordinates[atoms[1]-1])

    b1 = unit_vector(b1)
    b2 = unit_vector(b2)

    dot = np.dot(b1, b2)

    return np.rad2deg(np.arccos(dot))


def get_distance(coordinates, atoms):
    """
    Compute the distance between 2 atoms

    :param coordinates: list of coordinates of the molecule
    :param atoms: list of 2 atom indices to use to calculate the distance (from 1 to N)
    :return: the distance
    """

    b1 = np.array(coordinates[atoms[0]-1]) - np.array(coordinates[atoms[1]-1])

    return np.linalg.norm(b1)




from pyqchem import get_output_from_qchem, QchemInput, Structure
from pyqchem.parsers.basic import basic_parser_qchem
import numpy as np

molecule = Structure(coordinates=[[0.0, 0.0, 0.0],
                                  [0.0, 0.0, 0.9]],
                     symbols=['O', 'H'],
                     charge=-1,
                     multiplicity=1)

# create qchem input
qc_input = QchemInput(molecule,
                      jobtype='sp',
                      exchange='hf',
                      basis='6-31G',
                      unrestricted=True)

# calculate and parse qchem output

data = get_output_from_qchem(qc_input,
                             processors=4,
                             force_recalculation=True,
                             parser=basic_parser_qchem,
                             )


print('scf energy', data['scf_energy'], 'H')

print('Orbital energies (H)')
print('  {:^10} {:^10}'.format('alpha', 'beta'))
for a, b in zip(data['orbital_energies']['alpha'], data['orbital_energies']['beta']):
    print('{:10.3f} {:10.3f}'.format(a, b))

print('dipole moment:', data['multipole']['dipole_moment'], data['multipole']['dipole_units'])
print('quadrupole moment ({}):\n'.format(data['multipole']['quadrupole_units']),
      np.array(data['multipole']['quadrupole_moment']))
print('octopole moment ({}):\n'.format(data['multipole']['octopole_units']),
      np.array(data['multipole']['octopole_moment']))



from pyqchem.qchem_core import get_output_from_qchem
from pyqchem.qc_input import QchemInput
from pyqchem.parsers.parser_rasci import parser_rasci
from pyqchem.structure import Structure
from pyqchem.tools import rotate_coordinates

import numpy as np
import matplotlib.pyplot as plt

# define distances
distances = np.arange(0.0, np.pi+np.pi/21, np.pi/21)

energies = []
for dist in distances:

    # generate molecule
    coordinates = [[ 0.6695,  0.0000,  0.0000],
                   [-0.6695,  0.0000,  0.0000],
                   [ 1.2321,  0.9289,  0.0000],
                   [ 1.2321, -0.9289,  0.0000],
                   [-1.2321,  0.9289,  0.0000],
                   [-1.2321, -0.9289,  0.0000]]

    coordinates = rotate_coordinates(coordinates, angle=dist, axis=[1, 0, 0], atoms_list=[4, 5])

    ethene = Structure(coordinates=coordinates,
                       symbols=['C', 'C', 'H', 'H', 'H', 'H'],
                       charge=0,
                       multiplicity=1)

    # create qchem input
    qc_input = QchemInput(ethene,
                          jobtype='sp',
                          exchange='hf',
                          correlation='rasci',
                          basis='6-31G',
                          ras_act=4,
                          ras_elec=2,
                          ras_spin_mult=0,
                          ras_roots=2,
                          ras_do_part=False,
                          ras_do_hole=False,
                          ras_srdft_spinpol=True,
                          ras_omega=400,
                          max_scf_cycles=100,
                          ras_srdft_cor='srPBE',
                          #ras_srdft_exc='srPBE'
                          )

    # calculate and parse qchem output
    data, ee = get_output_from_qchem(qc_input,
                                     processors=3,
                                     parser=parser_rasci,
                                     read_fchk=True,
                                     # store_full_output=True,
                                     )

    guess = ee['coefficients']

    # store energies of excited states states in list
    energies.append([state['total_energy'] for state in data['excited_states']])

multiplicity_list = [state['multiplicity'] for state in data['excited_states']]

# transform energy list in array
energies = np.array(energies)

# print energies
print('\ndistance  state 1    state 2')
print('----------------------------')
for d, e in zip(distances, energies):
    print('{:4.1f} : {:10.5f} {:10.5f}'.format(d, e[0], e[1]))

# plot energies
for i, state_energy in enumerate(energies.T):
    plt.plot(distances, state_energy, label='state {} (S={})'.format(i+1, multiplicity_list[i]))

plt.title('Torsion of ethylene')
plt.xlabel('Torsion angle [Rad]')
plt.ylabel('Energy [Hartree]')

plt.legend()
plt.show()


from pyqchem.qchem_core import get_output_from_qchem
from pyqchem.qc_input import QchemInput
from pyqchem.parsers.parser_rasci import parser_rasci
from pyqchem.structure import Structure

import numpy as np
import matplotlib.pyplot as plt

# define distances
distances = np.arange(0.1, 3.5, 0.1)

energies = []
for dist in distances:

    # generate molecule
    molecule = Structure(coordinates=[[0.0, 0.0, 0.0],
                                      [0.0, 0.0, dist]],
                         symbols=['H', 'H'],
                         charge=0,
                         multiplicity=1)

    # create qchem input
    qc_input = QchemInput(molecule,
                          jobtype='sp',
                          exchange='hf',
                          correlation='rasci',
                          basis='sto-3g',
                          ras_act=2,
                          ras_elec=2,
                          ras_spin_mult=0,
                          ras_roots=2,
                          ras_do_part=False,
                          ras_do_hole=False)

    # calculate and parse qchem output
    data = get_output_from_qchem(qc_input,
                                 processors=4,
                                 parser=parser_rasci,
                                 store_full_output=True,
                                 )

    # store energies of excited states states in list
    energies.append([state['total_energy'] for state in data['excited_states']])

multiplicity_list = [state['multiplicity'] for state in data['excited_states']]

# transform energy list in array
energies = np.array(energies)

# print energies
print('\ndistance  state 1    state 2')
print('----------------------------')
for d, e in zip(distances, energies):
    print('{:4.1f} : {:10.5f} {:10.5f}'.format(d, e[0], e[1]))

# plot energies
for i, state_energy in enumerate(energies.T):
    plt.plot(distances, state_energy, label='state {} ({})'.format(i+1, multiplicity_list[i]))

plt.title('Dissociation of Hydrogen molecule')
plt.xlim([0, 3.5])
plt.ylim([-1.5, 1.0])
plt.xlabel('Distance [A]')
plt.ylabel('Energy [Hartree]')
plt.axhline(-1.0, color='grey', linestyle='--')

plt.legend()
plt.show()


from wfnsympy import WfnSympy
import numpy as np
from pyqchem.utils import get_occupied_electrons, reorder_coefficients
from pyqchem.utils import classify_diabatic_states_of_fragment, get_basis_functions_ranges_by_atoms
from pyqchem.utils import get_inertia, get_plane
from pyqchem import Structure


def get_wf_symmetry(structure,
                    basis,
                    mo_coeff,
                    center=None,
                    orientation=(0., 0., 1.),
                    orientation2=(0, 1, 0),
                    group='C2h',
                    occupancy=None):

    """
    Simplified interface between pyqchem anf wfnsympy

    :param structure: molecular geometry (3xn) array
    :param basis: dictionary containing the basis
    :param mo_coeff: molecular orbitals coefficients
    :param center: center of the molecule
    :param orientation: unit vector perpendicular to the plane of the molecule
    :param group: point symmetry group
    :return molsym: wfnsympy object
    """

    alpha_mo_coeff = np.array(mo_coeff['alpha']).tolist()

    alpha_occupancy = [0] * len(mo_coeff['alpha'])
    alpha_occupancy[:int(structure.alpha_electrons)] = [1] * structure.alpha_electrons

    if 'beta' in mo_coeff:
        beta_mo_coeff = np.array(mo_coeff['beta']).tolist()
        beta_occupancy = [0] * len(mo_coeff['beta'])
        beta_occupancy[:int(structure.alpha_electrons)] = [1] * structure.alpha_electrons
    else:
        beta_mo_coeff = None
        beta_occupancy = None

    if occupancy is not None:
        alpha_occupancy = occupancy['alpha']
        beta_occupancy = occupancy['alpha']

    molsym = WfnSympy(coordinates=structure.get_coordinates(),
                      symbols=structure.get_symbols(),
                      basis=basis,
                      center=center,
                      axis=orientation,
                      axis2=orientation2,
                      alpha_mo_coeff=alpha_mo_coeff,
                      beta_mo_coeff=beta_mo_coeff,
                      alpha_occupancy=alpha_occupancy,
                      beta_occupancy=beta_occupancy,
                      #charge=structure.charge,
                      #multiplicity=structure.multiplicity,
                      group=group)
    return molsym


def get_orbital_classification(fchk_data,
                               center=None,
                               orientation=(0., 0., 1.)):
    """
    Classify orbitals in sigma/pi in planar molecules

    :param fchk_data: dictionary containing fchk data
    :param orientation: unit vector perpendicular to the plane of the molecule
    :return: list of labels 'sigma'/'pi' according to the order of the orbitals
    """
    molsym = get_wf_symmetry(fchk_data['structure'],
                             fchk_data['basis'],
                             fchk_data['coefficients'],
                             center=center,
                             orientation=orientation)

    sh_index = molsym.SymLab.index('s_h')
    orbital_type_alpha = []
    for i, overlap in enumerate(molsym.mo_SOEVs_a[:, sh_index]):
        overlap = overlap / molsym.mo_SOEVs_a[i, molsym.SymLab.index('E')]  # normalize
        if overlap < 0:
            orbital_type_alpha.append(['pi', np.abs(overlap)])
        else:
            orbital_type_alpha.append(['sigma', np.abs(overlap)])

    if 'beta' in fchk_data['coefficients']:
        orbital_type_beta = []
        for i, overlap in enumerate(molsym.mo_SOEVs_b[:, sh_index]):
            if overlap < 0:
                orbital_type_beta.append(['pi', np.abs(overlap)])
            else:
                orbital_type_beta.append(['sigma', np.abs(overlap)])
        return orbital_type_alpha, orbital_type_beta
    else:
        return orbital_type_alpha


def get_state_symmetry(parsed_fchk,
                       rasci_states,
                       center=None,
                       orientation=(0, 0, 1),
                       orientation2=(0, 1, 0),
                       group='C2h',
                       extra_print=False,
                       amplitude_cutoff=0.0
                       ):
    """
    Determines electronic state symmetry (only for closed shell configurations)
    :param parsed_fchk: electronic structure
    :param rasci_states: parsed rasci output dictionary
    :param center: center of the molecule
    :param orientation:  main symmetry axis
    :param orientation2: secondary symmetry axis
    :param group: symmetry group label
    :param extra_print: if True prints wf & orbital symmetry analysis
    :param amplitude_cutoff: determined the configurations to take in account in the symmetry measure
    :return:
    """
    sym_states = {}
    for i, state in enumerate(rasci_states):

        # print('HF/KS Ground state',i ,'\n***********************')

        structure = parsed_fchk['structure']
        total_orbitals = len(parsed_fchk['coefficients']['alpha'])

        if extra_print:
            print('HF/KS Ground state\n---------------------------')
            molsym = get_wf_symmetry(structure,
                                     parsed_fchk['basis'],
                                     parsed_fchk['coefficients'],
                                     center=center,
                                     orientation=orientation,
                                     orientation2=orientation2,
                                     group=group)

            molsym.print_alpha_mo_IRD()
            molsym.print_beta_mo_IRD()
            molsym.print_wf_mo_IRD()

            print('\nRASCI Exited state {}\n---------------------------'.format(i+1))


        # print(configurations)
        occupations_list = []
        for configuration in state['configurations']:
            if np.abs(configuration['amplitude']) < amplitude_cutoff:
                continue

            occupations_list.append(configuration['occupations'])

            # print('occupied', occupied_orbitals, total_orbitals)
            # print('sum', np.sum(occupations_list[-1]['alpha']), np.sum(occupations_list[-1]['beta']))
            # print('alpha', occupations_list[-1]['alpha'])
            # print('beta', occupations_list[-1]['beta'])

            # print(configuration['hole'],'|' , configuration['alpha'], configuration['beta'], '|', configuration['part'])
            if extra_print:
                print('occ:', occupations_list[-1])
                print(configuration['alpha'], configuration['beta'], configuration['amplitude'])

        state_symmetry_list = []
        for occupations in occupations_list:
            # print('occupations', occupations)

            reordered_coefficients = reorder_coefficients(occupations, parsed_fchk['coefficients'])

            molsym = get_wf_symmetry(structure,
                                     parsed_fchk['basis'],
                                     reordered_coefficients,
                                     center=center,
                                     orientation=orientation,
                                     orientation2=orientation2,
                                     group=group)

            if extra_print:
                molsym.print_alpha_mo_IRD()
                molsym.print_beta_mo_IRD()
                molsym.print_wf_mo_IRD()

            state_symmetry = {}
            for label, measure in zip(molsym.IRLab, molsym.wf_IRd):
                state_symmetry[label] = measure

            # return maximum symmetry and value
            if extra_print:
                print([molsym.IRLab[np.argmax(molsym.wf_IRd)], np.max(molsym.wf_IRd)])
            state_symmetry_list.append([molsym.IRLab[np.argmax(molsym.wf_IRd)], np.max(molsym.wf_IRd)])

        if extra_print:
            print(state_symmetry_list)

        # Make sure symmetry of all configurations is the same
        if len(np.unique([a[0] for a in state_symmetry_list])) == 1:
            symmetry_label = state_symmetry_list[0][0]
            average_measure = np.average([a[1] for a in state_symmetry_list])
            sym_states['state {}'.format(i+1)] = [symmetry_label, average_measure]
        else:
            sym_states['state {}'.format(i+1)] = ['Undef', 0]

    return sym_states

def _indices_from_ranges(ranges):
    indices = []
    for i, j in ranges:
        indices += list(range(i, j))

    return indices


def get_symmetry_le(electronic_structure, data_rasci, fragment_atoms=(0), tol=0.1, group='D2h'):
    # This only works for singlets on close shell calculations

    types = classify_diabatic_states_of_fragment(data_rasci['diabatization']['diabatic_states'], fragment_atoms, tol=0.1)
    functions_range = get_basis_functions_ranges_by_atoms(electronic_structure['basis'], atoms_range=fragment_atoms)
    coordinates_frag = np.array(electronic_structure['structure'].get_coordinates())[fragment_atoms]
    labels_frag = electronic_structure['structure'].get_symbols()[fragment_atoms]

    inertia_moments, inertia_axis = get_inertia(Structure(coordinates=coordinates_frag, symbols=labels_frag))
    center_frag, normal_frag = get_plane(coordinates_frag)

    # print(np.array(inertia_axis))
    # print(inertia_moments)
    # print(inertia_axis[0])
    # print(inertia_axis[1])
    # print(inertia_axis[2])

    if 'LE' in types:
        indices = [i for i, x in enumerate(types) if x == "LE"]
        symmetry_labels = []
        for index in indices:
            print('\nLE state found at diabat {}!'.format(index+1))

            coefficients = electronic_structure['nato_coefficients_multi'][index+1]
            occupation = electronic_structure['nato_occupancies_multi'][index+1]
            overlap_matrix = np.array(electronic_structure['overlap'])

            functions_indices = _indices_from_ranges(functions_range)
            overlap_matrix = np.array([ovm[functions_indices] for ovm in overlap_matrix[functions_indices]])

            c_alpha = np.array(coefficients['alpha'])
            oc_alpha = np.array(occupation['alpha'])

            orbitals = []
            alpha = 0
            beta = 0

            coefficients_new = {'alpha': [], 'beta': []}
            for i, (va, o) in enumerate(zip(c_alpha, oc_alpha)):
                va_frag = va[functions_indices]
                frag_dot = np.dot(va_frag, np.dot(overlap_matrix, va_frag))

                if np.abs(frag_dot - 0.5) > tol:
                    if frag_dot > 0.5:
                        orbitals.append((i, np.round(o)))

                        orbital = np.zeros(len(c_alpha))
                        for j, val in zip(functions_indices, va_frag):
                            orbital[j] = val/np.sqrt(frag_dot)

                        if np.abs(o - 1) < tol and o < 2 - tol:
                            if alpha == beta:
                                alpha += 1
                                coefficients_new['alpha'].append(list(orbital))
                            else:
                                beta += 1
                                coefficients_new['beta'].append(list(orbital))
                    else:
                        pass

            if alpha == beta:
                multiplicity = 1
            else:
                multiplicity = 3

            n_electrons = alpha + beta

            try:
                # This only works for singlets
                molsym = get_wf_symmetry(electronic_structure['structure'],
                                         electronic_structure['basis'],
                                         coefficients_new,
                                         center=center_frag,
                                         orientation=inertia_axis[0],
                                         orientation2=inertia_axis[1],
                                         group=group
                                         )

                #molsym.print_alpha_mo_IRD()
                #molsym.print_beta_mo_IRD()
                molsym.print_wf_mo_IRD()
                symmetry_labels.append(molsym.IRLab[np.argmax(molsym.wf_IRd)])
            except:
                pass

        return symmetry_labels

    return None

if __name__ == '__main__':
    from pyqchem.qchem_core import get_output_from_qchem, create_qchem_input
    from pyqchem.structure import Structure
    from pyqchem.file_io import build_fchk
    from pyqchem.utils import _set_zero_to_coefficients

    benzene_coordinates = [[ 0.00000,  1.40272, 0.00000],
                           [ 0.00000,  2.49029, 0.00000],
                           [-1.21479,  0.70136, 0.00000],
                           [-2.15666,  1.24515, 0.00000],
                           [-1.21479, -0.70136, 0.00000],
                           [-2.15666, -1.24515, 0.00000],
                           [ 0.00000, -1.40272, 0.00000],
                           [ 0.00000, -2.49029, 0.00000],
                           [ 1.21479, -0.70136, 0.00000],
                           [ 2.15666, -1.24515, 0.00000],
                           [ 1.21479,  0.70136, 0.00000],
                           [ 2.15666,  1.24515, 0.00000]]

    symbols = ['C','H','C','H','C','H','C','H','C','H','C','F']

    benzene_coordinates = np.array(benzene_coordinates)
    molecule = Structure(coordinates=benzene_coordinates,
                         symbols=symbols,
                         charge=0)

    parameters = {'jobtype': 'sp',
                  'exchange': 'hf',
                  'basis': '6-31G'}

    qc_input = create_qchem_input(molecule, **parameters)

    _, _, fchk_data = get_output_from_qchem(qc_input, processors=4, force_recalculation=True,
                                            read_fchk=True)

    from pyqchem.file_io import build_fchk
    open('test.fchk', 'w').write(build_fchk(fchk_data))

    mo_coeff = _set_zero_to_coefficients(fchk_data['basis'],
                                         fchk_data['coefficients'],
                                         range(6, 12))
    fchk_data['coefficients'] = mo_coeff

    structure = fchk_data['structure']
    print(structure.get_xyz())

    txt_fchk = build_fchk(fchk_data)
    open('test2.fchk', 'w').write(txt_fchk)

    z_dist = structure.get_coordinates()[0][2]

    orbital_type = get_orbital_classification(fchk_data, center=[0.0, 0.0, z_dist])

    print('  {:5} {:5} {:5}'.format('num', 'type', 'overlap'))
    for i, ot in enumerate(orbital_type):
        print('{:5}:  {:5} {:5.2f}'.format(i+1, ot[0], ot[1]))



from pyqchem.qchem_core import get_output_from_qchem, create_qchem_input, redefine_calculation_data_filename
from pyqchem.parsers.parser_rasci_basic import basic_rasci
from pyqchem.parsers.parser_optimization import basic_optimization
from pyqchem.structure import Structure
from pyqchem.test import standardize_dictionary
import yaml
import unittest
import os, sys


redefine_calculation_data_filename('test_data.db')

if 'USER' in os.environ and os.environ['USER'] == 'travis':
    recalculate = False
else:
    recalculate = True


class HydrogenTest(unittest.TestCase):

    def setUp(self):
        self.assertDictEqual.__self__.maxDiff = None

        # generate molecule
        self.molecule = Structure(coordinates=[[0.0, 0.0, 0.0],
                                               [0.0, 0.0, 1.5]],
                                  symbols=['H', 'H'],
                                  charge=0,
                                  multiplicity=1)

    def test_srdft(self):

        # create qchem input
        txt_input = create_qchem_input(self.molecule,
                                       jobtype='sp',
                                       exchange='hf',
                                       correlation='rasci',
                                       basis='6-31G(d,p)',
                                       ras_act=2,
                                       ras_elec=2,
                                       ras_spin_mult=0,
                                       ras_roots=2,
                                       ras_do_hole=True,
                                       ras_sts_tm=True,
                                       # rasci sr-dft
                                       ras_omega=400,
                                       ras_srdft_cor='srpw92',
                                       ras_srdft_exc='srpbe',
                                       ras_natorb=False,
                                       set_iter=30,
                                       ras_srdft_damp=0.5)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input,
                                       processors=4,
                                       force_recalculation=recalculate,
                                       store_full_output=True)
        print(output)
        data = basic_rasci(output)

        filename = self.__class__.__name__ + '_srdft.yaml'

        #with open(filename, 'w') as outfile:
        #      yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        data = standardize_dictionary(data)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded)

        self.assertDictEqual(data, data_loaded)


    def test_rasci(self):
        # create qchem input
        txt_input = create_qchem_input(self.molecule,
                                       jobtype='sp',
                                       exchange='hf',
                                       correlation='rasci',
                                       basis='sto-3g',
                                       ras_act=2,
                                       ras_elec=2,
                                       ras_spin_mult=0,
                                       ras_roots=2,
                                       ras_print=5,
                                       ras_do_hole=True,
                                       ras_sts_tm=True)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input,
                                       processors=4,
                                       force_recalculation=recalculate,
                                       store_full_output=True)
        print(output)
        data = basic_rasci(output)

        filename = self.__class__.__name__ + '_rasci.yaml'

        # with open(filename, 'w') as outfile:
        #      yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        data = standardize_dictionary(data)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded)

        self.assertDictEqual(data, data_loaded)


class WaterTest(unittest.TestCase):

    def setUp(self):
        self.assertDictEqual.__self__.maxDiff = None

        # generate molecule
        molecule = Structure(coordinates=[[0.0, 1.0, 0.0],
                                          [0.0, 0.0, 1.0],
                                          [0.0, 0.0, -1.0]],
                             symbols=['O', 'H', 'H'],
                             charge=0,
                             multiplicity=1)

        # optimization
        txt_input = create_qchem_input(molecule,
                                       jobtype='opt',
                                       exchange='hf',
                                       basis='sto-3g',
                                       geom_opt_tol_gradient=300,
                                       geom_opt_tol_energy=100,
                                       geom_opt_coords=-1,
                                       geom_opt_tol_displacement=1200)

        parsed_data = get_output_from_qchem(txt_input,
                                            processors=4,
                                            parser=basic_optimization,
                                            force_recalculation=recalculate,
                                            store_full_output=True)

        self.molecule = parsed_data['optimized_molecule']

    def test_rasci(self):
        # create qchem input
        txt_input = create_qchem_input(self.molecule,
                                       jobtype='sp',
                                       exchange='hf',
                                       correlation='rasci',
                                       basis='sto-3g',
                                       ras_act=4,
                                       ras_elec=4,
                                       ras_spin_mult=1,
                                       ras_roots=6,
                                       ras_print=5,
                                       ras_do_hole=True,
                                       ras_sts_tm=True)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input,
                                       processors=4,
                                       force_recalculation=recalculate,
                                       store_full_output=True)
        print(output)
        data = basic_rasci(output)

        print(data)

        filename = self.__class__.__name__ + '_rasci.yaml'

        # with open(filename, 'w') as outfile:
        #     yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        data = standardize_dictionary(data)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded)

        self.assertDictEqual(data, data_loaded)


    def test_srdft(self):

        # create qchem input
        txt_input = create_qchem_input(self.molecule,
                                       jobtype='sp',
                                       exchange='hf',
                                       correlation='rasci',
                                       basis='6-31G(d,p)',
                                       ras_act=4,
                                       ras_elec=4,
                                       ras_spin_mult=0,
                                       ras_roots=6,
                                       ras_do_hole=True,
                                       ras_sts_tm=True,
                                       # rasci sr-dft
                                       ras_omega=300,
                                       ras_srdft_cor='srpbe',
                                       ras_srdft_exc='srlsda',
                                       ras_natorb=False,
                                       set_iter=30,
                                       ras_srdft_damp=0.5)

        print(txt_input.get_txt())

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input,
                                       processors=4,
                                       force_recalculation=recalculate,
                                       store_full_output=True)

        print(output)
        data = basic_rasci(output)

        print(data)

        filename = self.__class__.__name__ + '_srdft.yaml'

        # with open(filename, 'w') as outfile:
        #     yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        data = standardize_dictionary(data)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded)

        self.assertDictEqual(data, data_loaded)



from pyqchem.qchem_core import get_output_from_qchem, create_qchem_input
from pyqchem.parsers.basic import basic_parser_qchem
from pyqchem.structure import Structure
import numpy as np


# define molecule
coordinates = [[ 0.00000, 1.40272, 0.00000],
               [ 0.00000, 2.49029, 0.00000],
               [-1.21479, 0.70136, 0.00000],
               [-2.15666, 1.24515, 0.00000],
               [-1.21479, -0.70136, 0.00000],
               [-2.15666, -1.24515, 0.00000],
               [ 0.00000, -1.40272, 0.00000],
               [ 0.00000, -2.49029, 0.00000],
               [ 1.21479, -0.70136, 0.00000],
               [ 2.15666, -1.24515, 0.00000],
               [ 1.21479, 0.70136, 0.00000],
               [ 2.15666, 1.24515, 0.00000]]

symbols = ['C', 'H', 'C', 'H', 'C', 'H', 'C', 'H', 'C', 'H', 'C', 'H']

molecule = Structure(coordinates=coordinates,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

print('Initial structure')
print(molecule)

# optimization
qc_input = create_qchem_input(molecule,
                              jobtype='sp',
                              exchange='hf',
                              basis='sto-3g',
                              unrestricted=True,
                              solvent_method='pcm',
                              solvent_params={'Dielectric': 8.93},  # Cl2CH2
                              pcm_params={'Theory': 'CPCM',
                                          'Method': 'SWIG',
                                          'Solver': 'Inversion',
                                          'Radii': 'Bondi'}
                              )


data, electronic_structure = get_output_from_qchem(qc_input,
                                                   force_recalculation=True,
                                                   processors=4,
                                                   parser=basic_parser_qchem,
                                                   read_fchk=True)


print('scf energy', data['scf_energy'], 'H')

print('Orbital energies (H)')
print('  {:^10} {:^10}'.format('alpha', 'beta'))
for a, b in zip(data['orbital_energies']['alpha'], data['orbital_energies']['beta']):
    print('{:10.3f} {:10.3f}'.format(a, b))

print('dipole moment:', data['multipole']['dipole_moment'], data['multipole']['dipole_units'])
print('quadrupole moment ({}):\n'.format(data['multipole']['quadrupole_units']),
      np.array(data['multipole']['quadrupole_moment']))
print('octopole moment ({}):\n'.format(data['multipole']['octopole_units']),
      np.array(data['multipole']['octopole_moment']))



from pyqchem.qchem_core import get_output_from_qchem, redefine_calculation_data_filename
from pyqchem.qc_input import QchemInput
from pyqchem.structure import Structure
from pyqchem.parsers.parser_rasci import parser_rasci as parser_rasci
from pyqchem.basis import get_basis_from_ccRepo, trucate_basis, basis_to_txt
import numpy as np
from pyqchem.errors import OutputError


redefine_calculation_data_filename('test_soc3s_b.pkl')

as_f_doblet = [[4, 3], 4, 1]
atom_f = Structure(coordinates=[[0.0, 0.0, 0.0]],
                   symbols=['F'],
                   charge=-1,
                   multiplicity=1,
                   name='F')

as_cl_doblet = [[8, 7], 8, 1]
atom_cl = Structure(coordinates=[[0.0, 0.0, 0.0]],
                    symbols=['Cl'],
                    charge=-1,
                    multiplicity=1,
                    name='Cl')


experimental = {'F': [0, 0],
                'Cl': [0,0]}

use_experimental = False

for atom, active_spaces in [(atom_cl, as_cl_doblet), (atom_f, as_f_doblet)]:
    for basis_name in ['cc-pCVTZ']:
        basis_custom_repo = get_basis_from_ccRepo(atom, basis_name)

        basis_custom_repo = trucate_basis(basis_custom_repo,
                                          shells=['G', 'H', 'I', 'J', 'K'])

        qc_input_d = QchemInput(atom,
                                jobtype='sp',
                                exchange='hf',
                                correlation='rasci',
                                unrestricted='False',
                                thresh=14,
                                scf_convergence=11,
                                cis_convergence=9,
                                max_cis_cycles=150,
                                max_scf_cycles=100,
                                basis=basis_custom_repo,
                                ras_elec_alpha=active_spaces[0][0],
                                ras_elec_beta=active_spaces[0][1],
                                ras_act=active_spaces[1],
                                ras_occ=active_spaces[2],
                                ras_spin_mult=0,
                                ras_roots=10,
                                calc_soc=1,
                                n_frozen_core=0,
                                # ras_do_part=False,
                                set_iter=1000,
                                mem_total=15000,
                                mem_static=900
                                )

        output_d = get_output_from_qchem(qc_input_d,
                                         processors=4,
                                         force_recalculation=False,
                                         # parser=parser_rasci,
                                         store_full_output=True
                                         )
        # print(output_s)
        with open('qchem_output_{}.out'.format(atom.name), 'w') as f:
            f.write(output_d)

        output_d = parser_rasci(output_d)

        print('\n---------------------------------------------')
        print('Atom: {}'.format(atom.name))
        print('Basis: {}'.format(basis_name))
        print('Active space (ele, act, occ): {}'.format(active_spaces))
        print('---------------------------------------------')

        for i, state in enumerate(output_d['excited_states']):
            print('Energy state {} ({}):  {: 18.12f} au'.format(i+1, state['multiplicity'], state['total_energy']))
            print(' Alpha  Beta   Amplitude')
            for j, conf in enumerate(state['configurations']):
                print('  {}  {} {:8.3f}'.format(conf['alpha'], conf['beta'], conf['amplitude']))

        print('\nsoc_tot (cm-1) [imaginary part]')
        print('   ' + ''.join(['{:^18}'.format(n) for n in range(1, 11)]))
        for i in range(1, 11):
            line = '{:3}'.format(i)
            for j in range(1, 11):
                try:
                    line += '{:18.12f}'.format(np.array(output_d['interstate_properties'][(i, j)]['total_soc_mat'])[0, 0].imag)
                except KeyError:
                    line += '        -         '
            print(line)

        print('Frozen occupied: {} / virtual: {}'.format(output_d['rasci_dimensions']['frozen_occupied'],
                                                         output_d['rasci_dimensions']['frozen_virtual']))


        np.set_printoptions(precision=6, suppress=True)
        coupling = np.zeros((6, 6), dtype=complex)
        for i in range(3):
            for j in range(3):
                if i != j:
                    for ms_i in range(2):
                        for ms_j in range(2):
                            # print(output_t['interstate_properties'])
                            # print(np.array(output_d['interstate_properties'][(i+1, j+1)]['total_soc_mat']).shape, '  :  ', [ms_i, ms_j])
                            coupling[i*2+ms_i, j*2+ms_j] = np.array(output_d['interstate_properties'][(i+1, j+1)]['total_soc_mat'])[ms_i, ms_j]

        if use_experimental:
            for i in range(5):
                coupling[i + 9, i + 9] = experimental[atom.name][0]
            coupling[14, 14] = experimental[atom.name][1]


        print('\n Couplings matrix (cm-1)')
        t_labels = ' '.join(np.array([['D{:2}(ms={:2})           '.format(j, i) for i in [1/2, -1/2]] for j in ['1', '2', '3']]).flatten())
        print('           {}'.format(t_labels))

        print(np.array2string(coupling, max_line_width=10000))

        print('\n Diagonalizated couplings matrix (cm-1)')

        eval = np.linalg.eigvalsh(coupling)

        print(np.array2string(np.diag(eval), max_line_width=10000))

        print('\n***********************************************************************\n')



from pyqchem import Structure, QchemInput, get_output_from_qchem
from pyqchem.parsers.parser_optimization import basic_optimization
from pyqchem.parsers.parser_cis import basic_cis
from pyqchem.plots import plot_diabatization, plot_state
from pyqchem.utils import get_ratio_of_condition

import numpy as np
import matplotlib.pyplot as plt


# define monomer
coor_monomer = [[ 0.6695,  0.0000,  0.0000],
                [-0.6695,  0.0000,  0.0000],
                [ 1.2321,  0.9289,  0.0000],
                [ 1.2321, -0.9289,  0.0000],
                [-1.2321,  0.9289,  0.0000],
                [-1.2321, -0.9289,  0.0000]]

symbols_monomer = ['C', 'C', 'H', 'H', 'H', 'H']

monomer = Structure(coordinates=coor_monomer,
                    symbols=symbols_monomer,
                    charge=0,
                    multiplicity=1)

# optimization qchem input
qc_input = QchemInput(monomer,
                      jobtype='opt',
                      exchange='hf',
                      basis='sto-3g',
                      geom_opt_tol_gradient=300,
                      geom_opt_tol_energy=100,
                      geom_opt_coords=-1,
                      geom_opt_tol_displacement=1200)

parsed_data = get_output_from_qchem(qc_input,
                                    processors=4,
                                    parser=basic_optimization)

opt_monomer = parsed_data['optimized_molecule']

print('Optimized monomer structure')
print(opt_monomer)

# Build dimer from monomer
coor_monomer2 = np.array(opt_monomer.get_coordinates())
coor_monomer2[:, 2] += 6.0  # monomers separation

coordinates = opt_monomer.get_coordinates() + coor_monomer2.tolist()
symbols_dimer = symbols_monomer * 2

dimer = Structure(coordinates=coordinates,
                  symbols=symbols_dimer,
                  charge=0,
                  multiplicity=1)

print('Dimer structure')
print(dimer)


# CIS qchem input
qc_input = QchemInput(dimer,
                      exchange='hf',
                      basis='sto-3g',
                      #unrestricted=True,
                      cis_n_roots=8,
                      cis_convergence=8,
                      cis_singlets=True,
                      cis_triplets=False,
                      cis_ampl_anal=True,
                      RPA=False,
                      mem_static=900,
                      set_iter=30
                      )

print(qc_input.get_txt())

parsed_data = get_output_from_qchem(qc_input,
                                    processors=4,
                                    force_recalculation=False,
                                    parser=basic_cis
                                    )


# Analysis of diabatic states to use in diabatization
print('\nStates to use in diabatization (1e, max_jump 3)')
list_diabatic = []
for i, state in enumerate(parsed_data['excited_states']):
    ratio = get_ratio_of_condition(state, n_electron=1, max_jump=3)
    if ratio > 0.9:
        list_diabatic.append(i+1)
        mark = 'X'
    else:
        mark = ''
    print('State {}: {:4.3f}  {}'.format(i+1, ratio, mark))


# RASCI qchem input
qc_input = QchemInput(dimer,
                      exchange='hf',
                      basis='sto-3g',
                      #unrestricted=True,
                      cis_n_roots=8,
                      cis_convergence=8,
                      cis_singlets=True,
                      cis_triplets=False,
                      cis_ampl_anal=True,
                      loc_cis_ov_separate=False,
                      namd_nsurfaces=0,
                      er_cis_numstate=len(list_diabatic),
                      cis_diabath_decompose=True,
                      localized_diabatization=list_diabatic,
                      RPA=False,
                      mem_static=900,
                      set_iter=30
                      )

# print(qc_input.get_txt())

parsed_data = get_output_from_qchem(qc_input,
                                    processors=4,
                                    force_recalculation=False,
                                    parser=basic_cis,
                                    store_full_output=True,
                                    )

# parsed_data = rasci_parser(parsed_data)
# print(parsed_data)

# print adiabatic states
print('\nAdiabatic states dimer\n--------------------')
for i, state in enumerate(parsed_data['excited_states']):
    print('\nState {}'.format(i+1))
    print('Transition DM: ', state['transition_moment'])
    print('Energy: ', state['excitation_energy'])
    print(' Origin  Target   Amplitude')
    for j, conf in enumerate(state['configurations']):
        print('{:^9} {:^7} {:8.3f}'.format(conf['origin'], conf['target'], conf['amplitude']))

# plot adiabatic states
for i, state in enumerate(parsed_data['excited_states']):
    plot_state(state, with_amplitude=True, orbital_range=[dimer.alpha_electrons-4, dimer.alpha_electrons+4])
    plt.title('Adiabatic State {}'.format(i+1))
plt.show()

# diabatization analysis
diabatization = parsed_data['diabatization']

print('\nRotation Matrix')
print(np.array(diabatization['rot_matrix']))

print('\nAdiabatic Matrix')
print(np.array(diabatization['adiabatic_matrix']))

print('\nDiabatic Matrix')
print(np.array(diabatization['diabatic_matrix']))


print('\nDiabatic states dimer\n--------------------')

for i, state in enumerate(diabatization['diabatic_states']):
    print('\nState {}'.format(i+1))
    mat = [state['transition_moment'] for i, state in enumerate(parsed_data['excited_states']) if i+1 in list_diabatic]
    tdm = np.dot(np.array(diabatization['rot_matrix']).T, mat)[i]
    print('Transition DM: ', tdm)
    print('Energy: ', state['excitation_energy'])

plot_diabatization(diabatization['diabatic_states'], atoms_ranges=[dimer.get_number_of_atoms()/2,
                                                                   dimer.get_number_of_atoms()])


# Monomer adiabatic states (extra test)
qc_input = QchemInput(dimer,
                      #method='cis',
                      exchange='hf',
                      basis='sto-3g',
                      #unrestricted=True,
                      cis_n_roots=10,
                      cis_convergence=8,
                      cis_singlets=True,
                      cis_triplets=False,
                      cis_ampl_anal=True,
                      RPA=False,
                      mem_static=900,
                      set_iter=30
                      )

parsed_data = get_output_from_qchem(qc_input,
                                    processors=14,
                                    force_recalculation=False,
                                    parser=basic_cis
                                    )

print('\nAdiabatic states monomer\n--------------------')
for i, state in enumerate(parsed_data['excited_states']):
    print('\nState {}'.format(i+1))
    print('Transition DM: ', state['transition_moment'])
    print('Energy: ', state['excitation_energy'])
    print(' Origin  Target   Amplitude')
    for j, conf in enumerate(state['configurations']):
        print('{:^9} {:^7} {:8.3f}'.format(conf['origin'], conf['target'], conf['amplitude']))



from pyqchem.parsers.parser_rasci import parser_rasci as rasci_parser
from pyqchem import get_output_from_qchem, Structure, QchemInput
from pyqchem.file_io import build_fchk
from pyqchem.symmetry import get_symmetry_le
from pyqchem.qchem_core import redefine_calculation_data_filename
from pyqchem.utils import get_plane
from pyqchem.utils import get_ratio_of_condition
from pyqchem.tools import print_excited_states, plot_rasci_state_configurations
from pyqchem.plots import plot_diabatization
import numpy as np


redefine_calculation_data_filename('diabatic_naphtalene.pkl')


# define dimer
coor_monomer1 = [[ 2.4610326539,  0.7054950347, -0.0070507104],
                 [ 1.2697800226,  1.4213478618,  0.0045894884],
                 [ 0.0071248839,  0.7134976955,  0.0071917580],
                 [-1.2465927908,  1.4207541565,  0.0039025332],
                 [ 2.4498274919, -0.7358510124,  0.0046346543],
                 [ 3.2528295760,  1.2280710625, -0.0312673955],
                 [ 1.3575083440,  2.3667492466,  0.0220260183],
                 [-1.2932627225,  2.3688000888, -0.0152164523],
                 [ 3.2670227933, -1.2176289251,  0.0251089819],
                 [-2.4610326539, -0.7054950347,  0.0070507104],
                 [-1.2697800226, -1.4213478618, -0.0045894884],
                 [-0.0071248839, -0.7134976955, -0.0071917580],
                 [ 1.2465927908, -1.4207541565, -0.0039025332],
                 [-2.4498274919,  0.7358510124, -0.0046346543],
                 [-3.2528295760, -1.2280710625,  0.0312673955],
                 [-1.3575083440, -2.3667492466, -0.0220260183],
                 [ 1.2932627225, -2.3688000888,  0.0152164523],
                 [-3.2670227933,  1.2176289251, -0.0251089819]]

# set dimer geometry
coor_monomer2 = np.array(coor_monomer1).copy()
coor_monomer2[:, 2] += 4.0
# coor_monomer2[:, 1] -= 5.0
coor_monomer2 = list(coor_monomer2)

symbols_monomer = ['C', 'C', 'C', 'C', 'C', 'H', 'H', 'H', 'H',
                   'C', 'C', 'C', 'C', 'C', 'H', 'H', 'H', 'H']

coordinates = coor_monomer1 + coor_monomer2
symbols_dimer = symbols_monomer * 2

dimer = Structure(coordinates=coordinates,
                  symbols=symbols_dimer,
                  charge=0,
                  multiplicity=1)

range_f1 = range(len(coor_monomer1))
range_f2 = range(len(coor_monomer1), len(coordinates))

print('Dimer structure')
print(dimer)

# RASCI qchem input
qc_input = QchemInput(dimer,
                      unrestricted=False,
                      jobtype='sp',
                      exchange='hf',
                      correlation='rasci',
                      basis='sto-3g',
                      ras_act=4,
                      ras_elec=4,
                      # ras_occ=66,
                      ras_spin_mult=1,  # singlets only
                      ras_roots=10,      # calculate 8 states
                      ras_do_hole=True,
                      ras_do_part=True,
                      # ras_srdft_cor='srpbe',
                      # ras_srdft_exc='srpbe',
                      # ras_omega=300,
                      set_iter=60)


parsed_data, electronic_structure = get_output_from_qchem(qc_input,
                                                          processors=4,
                                                          force_recalculation=False,
                                                          parser=rasci_parser,
                                                          read_fchk=True,
                                                          store_full_output=True
                                                          )

print('final structure')
print(electronic_structure['structure'])

# Analysis of diabatic states to use in diabatization
print('\nAdiabatic states to use in diabatization (1e, max_jump 4)')
list_diabatic = []
for i, state in enumerate(parsed_data['excited_states']):
    ratio = get_ratio_of_condition(state, n_electron=1, max_jump=4)

    if ratio > 0.1:
        mark = 'X'
        list_diabatic.append(i+1)
    else:
        mark = ''

    print('State {}: {:4.3f}  {}'.format(i+1, ratio, mark))


# sequential diabatization scheme (2 steps)
diabatization_scheme = [
                        # {'method': 'ER', 'states': list(range(1, len(list_diabatic)+1))},
                        {'method': 'Boys', 'states': list(range(1, len(list_diabatic)+1))},
                        # {'method': 'ER', 'states': [3, 4, 5, 6]},
                        # {'method': 'DQ', 'states': list(range(1, len(list_diabatic)+1)), 'paramters': [0.9]}
                        ]

# RASCI qchem input
qc_input = QchemInput(electronic_structure['structure'],
                      jobtype='sp',
                      exchange='hf',
                      correlation='rasci',
                      basis='sto-3g',
                      scf_guess=electronic_structure['coefficients'],
                      ras_act=4,
                      ras_elec=4,
                      # ras_occ=66,
                      ras_spin_mult=1,  # singlets only
                      ras_roots=10,      # calculate 8 states
                      ras_do_hole=True,
                      ras_do_part=True,
                      # ras_srdft_cor='srpbe',
                      # ras_srdft_exc='srpbe',
                      # ras_omega=300,
                      ras_diabatization_states=list_diabatic,  # adiabatic states selected for diabatization
                      ras_diabatization_scheme=diabatization_scheme,
                      set_iter=60)

# print(qc_input.get_txt())

parsed_data, electronic_structure = get_output_from_qchem(qc_input,
                                                          processors=14,
                                                          force_recalculation=False,
                                                          parser=rasci_parser,
                                                          read_fchk=True,
                                                          store_full_output=True
                                                          )

#print(parsed_data)
#exit()
np.set_printoptions(precision=3)

print('\nAdiabatic states\n--------------------')
print_excited_states(parsed_data['excited_states'])

#plot_rasci_state_configurations(parsed_data['excited_states'])

# diabatization analysis
diabatization = parsed_data['diabatization']

print('\nRotation Matrix')
print(np.array(diabatization['rot_matrix']))

print('\nAdiabatic Matrix')
print(np.array(diabatization['adiabatic_matrix']))

print('\nDiabatic Matrix')
print(np.array(diabatization['diabatic_matrix']))

print('\nDiabatic states\n--------------------')
print_excited_states(diabatization['diabatic_states'], include_mulliken_rasci=True)


plot_diabatization(diabatization['diabatic_states'], atoms_ranges=[dimer.get_number_of_atoms()/2,
                                                                   dimer.get_number_of_atoms()])

coordinates_f1 = np.array(electronic_structure['structure'].get_coordinates())[range_f1]
center_f1, normal_f1 = get_plane(coordinates_f1)

coordinates_f2 = np.array(electronic_structure['structure'].get_coordinates())[range_f2]
center_f2, normal_f2 = get_plane(coordinates_f2)

print('\nDiabatic states symmetry analysis (monomer 1)')
sym_data = get_symmetry_le(electronic_structure, parsed_data, fragment_atoms=range_f2, group='D2h')
print('Symmetry: ', sym_data)

print('\nDiabatic states symmetry analysis (monomer 2)')
sym_data = get_symmetry_le(electronic_structure, parsed_data, fragment_atoms=range_f1, group='D2h')
print('Symmetry: ', sym_data)



from pyqchem.qchem_core import get_output_from_qchem, create_qchem_input, redefine_calculation_data_filename
from pyqchem.parsers.parser_rasci_basic import basic_rasci
from pyqchem.structure import Structure
from pyqchem.test import standardize_dictionary

import yaml
import unittest

ctrl_print = False  # set to true to generate reference files
do_alpha_beta = False # explicitly defining alpha/beta number of electrons?


redefine_calculation_data_filename('test_data.db')


class Eth00(unittest.TestCase):

    def setUp(self):
        self.assertDictEqual.__self__.maxDiff = None

        # generate molecule
        self.molecule = Structure(coordinates=[
                                  [0.665000, 0.000000, 0.000000],
                                  [-0.665000, 0.000000, 0.000000],
                                  [-1.230407, 0.915473, 0.000000],
                                  [-1.230407, -0.915473, 0.000000],
                                  [1.230407, 0.915473, 0.000000],
                                  [1.230407, -0.915473, 0.000000]],
                  symbols=['C', 'C', 'H', 'H', 'H', 'H'],
                  charge=0,
                  multiplicity=1)

        self.rem = {'jobtype': 'sp',
                    'exchange': 'hf',
                    'basis': '6-31G(d,p)',
                    'thresh': 14,
                    'scf_convergence': 8,
                    'max_scf_cycles': 150,
                    'max_cis_cycles': 150,
                    'unrestricted': False,
                    # RASCI
                    'correlation': 'rasci',
                    'ras_act': 2,
                    'ras_elec': 2,
                    'ras_occ': 7,
                    'ras_spin_mult': 0,
                    'ras_roots': 7,
                    'ras_do_hole': True,
                    'ras_do_part': True,
                    'ras_sts_tm': True,
                    'ras_natorb': True,
                    # rasci sr-dft
                    # 'ras_srdft': False,
                    # 'ras_omega': 400,
                    # 'ras_srdft_cor': 'srpw92',
                    # 'ras_srdft_exc': 'srpbe',
                    # 'ras_srdft_damp': 0.5,
                    #  rasci print level
                    'ras_print': 2,
                    # Frozen
                    'n_frozen_core': 0,
                    'n_frozen_virt': 0,
                    'set_iter': 60}

    def test_eth_00_ras22(self):

        rasci = dict(self.rem)
        if do_alpha_beta:
            rasci.update({'ras_elec_alpha': 1,
                          'ras_elec_beta': 1})

        # create qchem input
        txt_input = create_qchem_input(self.molecule, **rasci)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input, processors=4)
        print(output)
        data = basic_rasci(output)

        filename = self.__class__.__name__ + '_ras22.yaml'

        # create reference file
        if ctrl_print:
            with open(filename, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        data = standardize_dictionary(data)

        #print(data)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded)

        self.assertDictEqual(data, data_loaded)

    def test_eth_00_ras44(self):

        rasci = dict(self.rem)
        rasci.update({'ras_act': 4,
                      'ras_elec': 4,
                      'ras_occ': 6})
        if do_alpha_beta:
            rasci.update({'ras_elec_alpha': 2,
                          'ras_elec_beta': 2})


        # create qchem input
        txt_input = create_qchem_input(self.molecule, **rasci)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input, processors=4)
        print(output)
        data = basic_rasci(output)

        print(data)

        filename = self.__class__.__name__ + '_ras44.yaml'

        # creat reference file
        if ctrl_print:
            with open(filename, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        data = standardize_dictionary(data)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded)

        self.assertDictEqual(data, data_loaded)

    def test_eth_00_ras66(self):

        rasci = dict(self.rem)
        rasci.update({'ras_act': 6,
                      'ras_elec': 6,
                      'ras_occ': 5})
        if do_alpha_beta:
            rasci.update({'ras_elec_alpha': 3,
                          'ras_elec_beta': 3})

        # create qchem input
        txt_input = create_qchem_input(self.molecule, **rasci)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input, processors=4)
        print(output)
        data = basic_rasci(output)

        print(data)

        filename = self.__class__.__name__ + '_ras66.yaml'

        # creat reference file
        if ctrl_print:
            with open(filename, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        data = standardize_dictionary(data)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded)

        self.assertDictEqual(data, data_loaded)


class Eth90(unittest.TestCase):

    def setUp(self):
        self.assertDictEqual.__self__.maxDiff = None

        # generate molecule
        self.molecule = Structure(coordinates=[
            [0.665000, 0.000000, 0.000000],
            [-0.665000, 0.000000, 0.000000],
            [-1.230407, 0.915473, 0.000000],
            [-1.230407, -0.915473, 0.000000],
            [1.230407, 0.0, 0.915473],
            [1.230407, 0.0, -0.915473]],
            symbols=['C', 'C', 'H', 'H', 'H', 'H'],
            charge=0,
            multiplicity=3)

        self.rem = {'jobtype': 'sp',
                    'exchange': 'hf',
                    'basis': '6-31G(d)',
                    'thresh': 14,
                    'scf_convergence': 8,
                    'max_scf_cycles': 150,
                    'max_cis_cycles': 150,
                    'unrestricted': False,
                    # RASCI
                    'correlation': 'rasci',
                    'ras_act': 2,
                    'ras_elec': 2,
                    'ras_occ': 7,
                    'ras_spin_mult': 0,
                    'ras_roots': 7,
                    'ras_do_hole': True,
                    'ras_do_part': True,
                    'ras_sts_tm': True,
                    'ras_natorb': True,
                    # rasci sr-dft
                    # 'ras_srdft': False,
                    # 'ras_omega': 400,
                    # 'ras_srdft_cor': 'srpw92',
                    # 'ras_srdft_exc': 'srpbe',
                    # 'ras_srdft_damp': 0.5,
                    # rasci print level
                    'ras_print': 2,
                    # Frozen
                    'n_frozen_core': 0,
                    'n_frozen_virt': 0,
                    'set_iter': 60}

    def test_eth_90_ras22(self):

        rasci = dict(self.rem)
        if do_alpha_beta:
            rasci.update({'ras_elec_alpha': 1,
                          'ras_elec_beta': 1})

        # create qchem input
        txt_input = create_qchem_input(self.molecule, **rasci)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input, processors=4)
        print(output)
        data = basic_rasci(output)

        print(data)

        filename = self.__class__.__name__ + '_ras22.yaml'

        # create reference file
        if ctrl_print:
            with open(filename, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        data = standardize_dictionary(data)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded)

        self.assertDictEqual(data, data_loaded)

    def test_eth_90_ras44(self):

        rasci = dict(self.rem)
        rasci.update({'ras_act': 4,
                      'ras_elec': 4,
                      'ras_occ': 6})
        if do_alpha_beta:
            rasci.update({'ras_elec_alpha': 2,
                          'ras_elec_beta': 2})

        # create qchem input
        txt_input = create_qchem_input(self.molecule, **rasci)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input, processors=4)
        print(output)
        data = basic_rasci(output)

        print(data)

        filename = self.__class__.__name__ + '_ras44.yaml'

        # creat reference file
        if ctrl_print:
            with open(filename, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        data = standardize_dictionary(data, decimal=3)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded, decimal=3)

        self.assertDictEqual(data, data_loaded)

    def test_eth_90_ras66(self):

        rasci = dict(self.rem)
        rasci.update({'ras_act': 6,
                      'ras_elec': 6,
                      'ras_occ': 5})
        if do_alpha_beta:
            rasci.update({'ras_elec_alpha': 3,
                          'ras_elec_beta': 3})

        # create qchem input
        txt_input = create_qchem_input(self.molecule, **rasci)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input, processors=4)
        print(output)
        data = basic_rasci(output)

        print(data)

        filename = self.__class__.__name__ + '_ras66.yaml'

        # create reference file
        if ctrl_print:
            with open(filename, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        data = standardize_dictionary(data, decimal=3)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded, decimal=3)

        self.assertDictEqual(data, data_loaded)


class EthDist(unittest.TestCase):

    def setUp(self):
        self.assertDictEqual.__self__.maxDiff = None

        # generate molecule
        self.molecule = Structure(coordinates=[
            [0.865000, 0.200000, 0.800000],
            [-0.665000, -0.300000, 0.500000],
            [-1.130407, 0.715473, 0.220000],
            [-1.430407, -0.915473, 0.300000],
            [1.230407, 0.815473, -0.4],
            [1.000407,-0.888888, 0.45]],
            symbols=['C', 'C', 'H', 'H', 'H', 'H'],
            charge=0,
            multiplicity=1)

        self.rem = {'jobtype': 'sp',
                    'exchange': 'hf',
                    'basis': '6-31G(d,p)',
                    'thresh': 14,
                    'scf_convergence': 8,
                    'max_scf_cycles': 150,
                    'max_cis_cycles': 150,
                    'unrestricted': False,
                    # RASCI
                    'correlation': 'rasci',
                    'ras_act': 2,
                    'ras_elec': 2,
                    'ras_occ': 7,
                    'ras_spin_mult': 0,
                    'ras_roots': 7,
                    'ras_do_hole': True,
                    'ras_do_part': True,
                    'ras_sts_tm': True,
                    'ras_natorb': True,
                    # rasci sr-dft
                    # 'ras_srdft': False,
                    # 'ras_omega': 400,
                    # 'ras_srdft_cor': 'srpw92',
                    # 'ras_srdft_exc': 'srpbe',
                    # 'ras_srdft_damp': 0.5,
                    # rasci print level
                    'ras_print': 2,
                    # Frozen
                    'n_frozen_core': 0,
                    'n_frozen_virt': 0,
                    'set_iter': 60}

    def test_eth_dist_ras22(self):

        rasci = dict(self.rem)
        if do_alpha_beta:
            rasci.update({'ras_elec_alpha': 1,
                          'ras_elec_beta': 1})

        # create qchem input
        txt_input = create_qchem_input(self.molecule, **rasci)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input, processors=4)
        print(output)
        data = basic_rasci(output)

        print(data)

        filename = self.__class__.__name__ + '_ras22.yaml'

        # creat reference file
        if ctrl_print:
            with open(filename, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        data = standardize_dictionary(data)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded)

        self.assertDictEqual(data, data_loaded)

    def test_eth_dist_ras44(self):

        rasci = dict(self.rem)
        rasci.update({'ras_act': 4,
                      'ras_elec': 4,
                      'ras_occ': 6})
        if do_alpha_beta:
            rasci.update({'ras_elec_alpha': 2,
                          'ras_elec_beta': 2})

        # create qchem input
        txt_input = create_qchem_input(self.molecule, **rasci)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input, processors=4)
        print(output)
        data = basic_rasci(output)

        print(data)

        filename = self.__class__.__name__ + '_ras44.yaml'

        # creat reference file
        if ctrl_print:
            with open(filename, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        data = standardize_dictionary(data)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded)

        self.assertDictEqual(data, data_loaded)

    def test_eth_dist_ras66(self):

        rasci = dict(self.rem)
        rasci.update({'ras_act': 6,
                      'ras_elec': 6,
                      'ras_occ': 5})
        if do_alpha_beta:
            rasci.update({'ras_elec_alpha': 3,
                          'ras_elec_beta': 3})

        # create qchem input
        txt_input = create_qchem_input(self.molecule, **rasci)

        # calculate and parse qchem output
        output = get_output_from_qchem(txt_input, processors=4)
        print(output)
        data = basic_rasci(output)

        print(data)

        filename = self.__class__.__name__ + '_ras66.yaml'

        # creat reference file
        if ctrl_print:
            with open(filename, 'w') as outfile:
                yaml.dump(data, outfile, default_flow_style=False, allow_unicode=True)

        with open(filename, 'r') as stream:
            data_loaded = yaml.safe_load(stream)

        data = standardize_dictionary(data)

        print(data_loaded)
        data_loaded = standardize_dictionary(data_loaded)

        self.assertDictEqual(data, data_loaded)



from pyqchem import get_output_from_qchem, Structure, QchemInput
from pyqchem.parsers.parser_optimization import basic_optimization
from pyqchem.parsers.basic import basic_parser_qchem


# define molecule
coordinates = [[0.0,  1.0,  0.0],
               [0.0,  0.0,  1.0],
               [0.0,  0.0, -1.0]]

symbols = ['O', 'H', 'H']

molecule = Structure(coordinates=coordinates,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

print('Initial structure')
print(molecule)

# Initial calculation with STO-3G basis
qc_input = QchemInput(molecule,
                      jobtype='opt',
                      exchange='hf',
                      basis='sto-3g',
                      )

parsed_data, ee = get_output_from_qchem(qc_input,
                                        processors=4,
                                        parser=basic_optimization,
                                        force_recalculation=False,
                                        read_fchk=True)

opt_molecule = parsed_data['optimized_molecule']

print('Optimized structure')
print(opt_molecule)
print('Final energy:', parsed_data['energy'])

# Precise calculation with larger 6-31G basis using previous MO as guess
qc_input = QchemInput(opt_molecule,
                      jobtype='sp',
                      exchange='hf',
                      basis='6-31g',
                      basis2=ee['basis'],
                      scf_guess=ee['coefficients']
                      )


parsed_data = get_output_from_qchem(qc_input,
                                    processors=4,
                                    parser=basic_parser_qchem,
                                    force_recalculation=True)

print('results: ', parsed_data)


from pyqchem.qchem_core import get_output_from_qchem, redefine_calculation_data_filename
from pyqchem.qc_input import QchemInput
from pyqchem.structure import Structure
from pyqchem.parsers.parser_rasci import parser_rasci as parser_rasci
from pyqchem.basis import get_basis_from_ccRepo, trucate_basis, basis_to_txt
from pyqchem.errors import OutputError
import numpy as np

# import pyqchem.qchem_core
# pyqchem.qchem_core.__calculation_data_filename__ = 'test.pkl'
redefine_calculation_data_filename('test_soc2.pkl')

# create molecule
as_clo = [[3, 2, 11], [5, 3, 10], [11, 6, 7], [13, 7, 6], [19, 10, 3]]
mol_clo = Structure(coordinates=[[0.0, 0.0, 0.0000],
                                 [0.0, 0.0, 1.5696]],
                    symbols=['Cl', 'O'],
                    charge=-1,
                    multiplicity=1,
                    name='ClO')

as_bro = [[3, 2, 20], [9, 5, 17], [13, 7, 15], [23, 12, 10]]
mol_bro = Structure(coordinates=[[0.0, 0.0, 0.0000],
                                 [0.0, 0.0, 1.7172]],
                    symbols=['Br', 'O'],
                    charge=-1,
                    multiplicity=1,
                    name='BrO')

as_ncs = [[3, 2, 13], [7, 4, 11], [11, 6, 9], [15, 8, 7]]
mol_ncs = Structure(coordinates=[[0.0, 0.0, 1.159],
                                 [0.0, 0.0, 0.000],
                                 [0.0, 0.0,-1.631]],
                    symbols=['N', 'C', 'S'],
                    charge=-1,
                    multiplicity=1,
                    name='NCS')

as_n2o = [[3, 2, 9], [11, 6, 5], [15, 8, 3]]
mol_n2o = Structure(coordinates=[[0.0, 0.0, 1.154],
                                 [0.0, 0.0, 0.000],
                                 [0.0, 0.0,-1.182]],
                    symbols=['N', 'N', 'O'],
                    charge=0,
                    multiplicity=1,
                    name='N2O')

as_cco = [[3, 2, 9], [5, 3, 8], [11, 6, 5], [15, 8, 3]]
mol_cco = Structure(coordinates=[[0.0, 0.0, 1.308],
                                 [0.0, 0.0, 0.000],
                                 [0.0, 0.0,-1.221]],
                    symbols=['C', 'C', 'O'],
                    charge=-2,
                    multiplicity=1,
                    name='CCO')

as_ccf = [[3, 2, 9], [5, 3, 8], [11, 6, 5], [15, 8, 3]]
mol_ccf = Structure(coordinates=[[0.0, 0.0, 1.271],
                                 [0.0, 0.0, 0.000],
                                 [0.0, 0.0,-1.276]],
                    symbols=['C', 'C', 'F'],
                    charge=-1,
                    multiplicity=1,
                    name='CCF')

as_cccl = [[3, 2, 13], [5, 3, 12], [11, 6, 9], [15, 8, 7]]
mol_cccl = Structure(coordinates=[[0.0, 0.0, 1.2964],
                                  [0.0, 0.0, 0.000],
                                  [0.0, 0.0,-1.6224]],
                     symbols=['C', 'C', 'Cl'],
                     charge=-1,
                     multiplicity=1,
                     name='CCCl')


# Generate basis name list
basis_name_list = []

for molecule, active_space_list in [(mol_clo, as_clo), (mol_bro, as_bro), (mol_ncs, as_ncs), (mol_n2o, as_n2o),
                                    (mol_cco, as_cco), (mol_ccf, as_ccf), (mol_cccl, as_cccl)]:
    for basis_name in ['cc-pCVTZ', 'cc-pCVQZ']:
        for active_space in active_space_list:
            basis_custom_repo = get_basis_from_ccRepo(molecule,
                                                      basis_name,
                                                      if_missing=basis_name.replace('cc-pC', 'cc-p'))
            basis_custom_repo = trucate_basis(basis_custom_repo,
                                              shells=['G', 'H', 'I', 'J', 'K'])

            qc_input = QchemInput(molecule,
                                  jobtype='sp',
                                  exchange='hf',
                                  correlation='rasci',
                                  purecart='1111',
                                  unrestricted='False',
                                  thresh=14,
                                  scf_convergence=8,
                                  # max_cis_cycles=150,
                                  basis=basis_custom_repo,
                                  ras_elec=active_space[0],
                                  ras_act=active_space[1],
                                  ras_occ=active_space[2],
                                  ras_spin_mult=0,
                                  ras_roots=2,  # calculate 2 states
                                  calc_soc=1,
                                  set_iter=1000,
                                  mem_total=15000,
                                  mem_static=900,
                                  n_frozen_core=0
                                  )

            # print(qc_input.get_txt())
            try:
                output = get_output_from_qchem(qc_input,
                                               processors=4,
                                               force_recalculation=False,
                                               parser=parser_rasci,
                                               store_full_output=True
                                               )
            except OutputError as e:
                print('---------------------------------------------')
                print('Molecule: {}'.format(molecule.name))
                print('basis: {}'.format(basis_name))
                print('Active space (ele, act, occ): {}'.format(active_space))
                print(e.error_lines)
                print('calculation_failed')
                continue
            # print(_)
            # print(output['interstate_properties'])

            gamma_total = output['interstate_properties'][(1, 2)]['gamma_total']
            soc_1e = np.array(output['interstate_properties'][(1, 2)]['1e_soc_mat'])
            soc_2e = np.array(output['interstate_properties'][(1, 2)]['2e_soc_mat'])
            soc_tot = np.array(output['interstate_properties'][(1, 2)]['total_soc_mat'])
            socc = output['interstate_properties'][(1, 2)]['mf_socc']

            energy_1 = output['excited_states'][0]['total_energy']
            energy_2 = output['excited_states'][1]['total_energy']

            print('---------------------------------------------')
            print('molecule: {}'.format(molecule.name))
            print('basis: {}'.format(basis_name))
            print('Active space (ele, act, occ): {}'.format(active_space))
            print('gamma_tot: {}'.format(gamma_total))
            print('Energy state1:  {: 18.12f} au'.format(energy_1))
            print('Energy state2:  {: 18.12f} au'.format(energy_2))
            print('soc_1e  {0.real: 10.3f} + {0.imag: 10.8f} cm-1'.format(soc_1e[0, 0]))
            print('soc_2e  {0.real: 10.3f} + {0.imag: 10.8f} cm-1'.format(soc_2e[0, 0]))
            print('soc_tot {0.real: 10.3f} + {0.imag: 10.8f} cm-1'.format(soc_tot[0, 0]))
            # print('soc_1e (cm-1)')
            # print('{0.real: 8.3e} + {0.imag: 10.8e}i    {1.real: 8.3e} + {1.imag: 10.8e}i'.format(*soc_1e[0]))
            # print('{0.real: 8.3e} + {0.imag: 10.8e}i    {1.real: 8.3e} + {1.imag: 10.8e}i'.format(*soc_1e[1]))
            # print('soc_2e (cm-1)')
            # print('{0.real: 8.3e} + {0.imag: 10.8e}i    {1.real: 8.3e} + {1.imag: 10.8e}i'.format(*soc_2e[0]))
            # print('{0.real: 8.3e} + {0.imag: 10.8e}i    {1.real: 8.3e} + {1.imag: 10.8e}i'.format(*soc_2e[1]))
            # print('soc_tot (cm-1)')
            # print('{0.real: 8.3e} + {0.imag: 10.8e}i    {1.real: 8.3e} + {1.imag: 10.8e}i'.format(*soc_tot[0]))
            # print('{0.real: 8.3e} + {0.imag: 10.8e}i    {1.real: 8.3e} + {1.imag: 10.8e}i'.format(*soc_tot[1]))

            print('SOCC: {: 18.12f} cm-1'.format(socc))
            print('Frozen occupied: {} / virtual: {}'.format(output['rasci_dimensions']['frozen_occupied'],
                                                             output['rasci_dimensions']['frozen_virtual']))

            print('Frozen occupied: {} / virtual: {}'.format(output['rasci_dimensions']['frozen_occupied'],
                                                             output['rasci_dimensions']['frozen_virtual']))



from pyqchem.qchem_core import get_output_from_qchem, create_qchem_input
from pyqchem.structure import Structure
from pyqchem.symmetry import get_orbital_classification
from pyqchem.file_io import build_fchk


# define coordinates
benzene_coordinates = [[ 0.00000,  1.40272,  0.00000],
                       [ 0.00000,  2.49029,  0.00000],
                       [-1.21479,  0.70136,  0.00000],
                       [-2.15666,  1.24515,  0.00000],
                       [-1.21479, -0.70136,  0.00000],
                       [-2.15666, -1.24515,  0.00000],
                       [ 0.00000, -1.40272,  0.00000],
                       [ 0.00000, -2.49029,  0.00000],
                       [ 1.21479, -0.70136,  0.00000],
                       [ 2.15666, -1.24515,  0.00000],
                       [ 1.21479,  0.70136,  0.00000],
                       [ 2.15666,  1.24515,  0.00000]]

symbols = ['C', 'H', 'C', 'H', 'C', 'H', 'C', 'H', 'C', 'H', 'C', 'H']

# create molecule
molecule = Structure(coordinates=benzene_coordinates,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

# define Q-Chem parameters
parameters = {'jobtype': 'sp',
              'exchange': 'hf',
              'basis': '6-31G'}

# create Q-Chem input
qc_input = create_qchem_input(molecule, **parameters)

# get data from Q-Chem calculation
output, parsed_fchk = get_output_from_qchem(qc_input,
                                            processors=4,
                                            force_recalculation=False,
                                            read_fchk=True,
                                            fchk_only=True)


# write .fchk file copy on disk (for checking purpose)
txt_fchk = build_fchk(parsed_fchk)
open('test_benzene.fchk', 'w').write(txt_fchk)

# get orbital type
orbital_types = get_orbital_classification(parsed_fchk,
                                           center=[0.0, 0.0, 0.0],
                                           orientation=[0.0, 0.0, 1.0])

# print results
print('  {:5} {:5} {:5}'.format('num', 'type', 'overlap'))
for i, ot in enumerate(orbital_types):
    print('{:5}:  {:5} {:5.3f}'.format(i + 1, ot[0], ot[1]))


import numpy as np

from pyqchem.symmetry import get_wf_symmetry
from pyqchem.utils import get_plane, crop_electronic_structure
from pyqchem.qchem_core import get_output_from_qchem, create_qchem_input
from pyqchem.structure import Structure
from pyqchem.file_io import build_fchk


# Define custom classification function
def get_custom_orbital_classification(parsed_fchk,
                                      center=None,
                                      orientation=(0, 0, 1)
                                      ):

    molsym = get_wf_symmetry(parsed_fchk['structure'],
                             parsed_fchk['basis'],
                             parsed_fchk['coefficients'],
                             center=center,
                             orientation=orientation)

    sh_index = molsym.SymLab.index('i')  # operation used to separate orbitals
    orbital_type = []
    for i, overlap in enumerate(molsym.mo_SOEVs_a[:, sh_index]):
        overlap = overlap / molsym.mo_SOEVs_a[i, molsym.SymLab.index('E')]  # normalize
        if overlap < 0:
            orbital_type.append([' NOO', np.abs(overlap)])
        else:
            orbital_type.append([' YES', np.abs(overlap)])
    return orbital_type


dimer_ethene = [[0.0,  0.0000,   0.65750],
                [0.0,  0.0000,  -0.65750],
                [0.0,  0.92281,  1.22792],
                [0.0, -0.92281,  1.22792],
                [0.0, -0.92281, -1.22792],
                [0.0,  0.92281, -1.22792],
                [3.7,  0.00000,  0.65750],
                [3.7,  0.00000, -0.65750],
                [3.7,  0.92281,  1.22792],
                [3.7, -0.92281,  1.22792],
                [3.7, -0.92281, -1.22792],
                [3.7,  0.92281, -1.22792]]

symbols = ['C', 'C', 'H', 'H', 'H', 'H', 'C', 'C', 'H', 'H', 'H', 'H']

range_f1 = range(0, 6)
range_f2 = range(6, 12)


# create molecule
molecule = Structure(coordinates=dimer_ethene,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

# create Q-Chem input
qc_input = create_qchem_input(molecule,
                              jobtype='sp',
                              exchange='hf',
                              basis='6-31G')

print(qc_input.get_txt())
# get data from Q-Chem calculation
output, electronic_structure = get_output_from_qchem(qc_input,
                                                     processors=4,
                                                     force_recalculation=False,
                                                     read_fchk=True,
                                                     fchk_only=True)

# store original fchk info in file
open('test.fchk', 'w').write(build_fchk(electronic_structure))

# get symmetry classification
electronic_structure_f1 = crop_electronic_structure(electronic_structure, range_f1)

# save test fchk file with new coefficients
open('test_f1.fchk', 'w').write(build_fchk(electronic_structure_f1))

# get plane from coordinates
coordinates_f1 = electronic_structure['structure'].get_coordinates(fragment=range_f1)
center_f1, normal_f1 = get_plane(coordinates_f1)

# get classified orbitals
orbital_type_f1 = get_custom_orbital_classification(electronic_structure_f1,
                                                    center=center_f1,
                                                    orientation=normal_f1)

# get plane from coordinates
coordinates_f2 = electronic_structure['structure'].get_coordinates(fragment=range_f2)
center_f2, normal_f2 = get_plane(coordinates_f2)

electronic_structure_f2 = crop_electronic_structure(electronic_structure, range_f2)

# save test fchk file with new coefficients
open('test_f2.fchk', 'w').write(build_fchk(electronic_structure_f2))

# get classified orbitals
orbital_type_f2 = get_custom_orbital_classification(electronic_structure_f2,
                                                    center=center_f2,
                                                    orientation=normal_f2)

# range of orbitals to show
frontier_orbitals = [12, 13,  14, 15, 16, 17, 18, 19, 20]

# Print results in table
print('Inversion center?')
print('index   fragment 1   fragment 2')
for i in frontier_orbitals:
    print(' {:4}  {:4}  {:4.3f}  {:4}  {:4.3f}'.format(i,
                                       orbital_type_f1[i-1][0], orbital_type_f1[i-1][1],
                                       orbital_type_f2[i-1][0], orbital_type_f2[i-1][1]))



from pyqchem.qchem_core import get_output_from_qchem
from pyqchem.qc_input import QchemInput
from pyqchem.structure import Structure
from pyqchem.parsers.parser_rasci import parser_rasci
from pyqchem.parsers.parser_cis import basic_cis as parser_cis
import numpy as np

# Carbon atom defintion
atom_c = Structure(coordinates=[[0.0, 0.0, 0.0]],
                   symbols=['C'],
                   charge=-2,
                   multiplicity=1,
                   name='C')

basis_name = '6-31G**'
# RAS_CI calculation
qc_input_rasci = QchemInput(atom_c,
                            jobtype='sp',
                            exchange='hf',
                            correlation='rasci',
                            unrestricted=True,
                            thresh=14,
                            scf_convergence=8,
                            max_cis_cycles=150,
                            max_scf_cycles=100,
                            basis=basis_name,
                            ras_elec_alpha=3,
                            ras_elec_beta=1,
                            ras_act=4,
                            ras_occ=1,
                            ras_spin_mult=0,
                            ras_roots=8,
                            calc_soc=1,
                            n_frozen_core=0,
                            )

# CIS calculation
qc_input_cis = QchemInput(atom_c,
                          jobtype='sp',
                          exchange='b3lyp',
                          basis=basis_name,
                          cis_n_roots=8,
                          cis_singlets=True,
                          cis_triplets=True,
                          cis_convergence=8,
                          calc_soc=1,
                          n_frozen_core=0,
                          )

for qc_input, parser in [(qc_input_cis, parser_cis), (qc_input_rasci, parser_rasci)]:

    output = get_output_from_qchem(qc_input,
                                   processors=4,
                                   force_recalculation=True,
                                   parser=parser,
                                   )

    # print calculation data
    for i, state in enumerate(output['excited_states']):
        print('Energy state {} ({}):  {:18.12f} au'.format(i+1, state['multiplicity'], state['total_energy']))
        print('Transition energy  {:4.8} eV'.format(state['excitation_energy']))
        if qc_input is qc_input_cis:
            print(' Origin  Target   Amplitude')
        else:
            print(' Alpha  Beta   Amplitude')

        for j, conf in enumerate(state['configurations']):
            try:
                print('     {}     {}  {:8.3f}'.format(conf['origin'], conf['target'], conf['amplitude']))
            except KeyError:
                print('  {}  {} {:8.3f}'.format(conf['alpha'], conf['beta'], conf['amplitude']))

    print('\n1e_soc_mat (cm-1) [(0,0) element, imaginary part]')
    print('   ' + ''.join(['{:^18}'.format(n) for n in range(1, 9)]))
    for i in range(1, 9):
        line = '{:3}'.format(i)
        for j in range(1, 9):
            try:
                line += '{:18.12f}'.format(np.array(output['interstate_properties'][(i, j)]['1e_soc_mat'])[0, 0].imag)
            except KeyError:
                line += '        -         '
        print(line)




from pyqchem import get_output_from_qchem, QchemInput
from pyqchem.parsers.parser_frequencies import basic_frequencies
from pyqchem.parsers.parser_optimization import basic_optimization
from pyqchem.structure import Structure

import numpy as np
import matplotlib.pyplot as plt


# define molecule
coordinates = [[0.0,  1.0,  0.0],
               [0.0,  0.0,  1.0],
               [0.0,  0.0, -1.0]]

symbols = ['O', 'H', 'H']

molecule = Structure(coordinates=coordinates,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

print('Initial structure')
print(molecule)

# optimization
qc_input = QchemInput(molecule,
                      jobtype='opt',
                      exchange='hf',
                      basis='sto-3g',
                      geom_opt_tol_gradient=300,
                      geom_opt_tol_energy=100,
                      geom_opt_coords=-1,
                      geom_opt_tol_displacement=1200)


parsed_data, electronic_structure = get_output_from_qchem(qc_input,
                                                          processors=4,
                                                          parser=basic_optimization,
                                                          read_fchk=True)


opt_molecule = parsed_data['optimized_molecule']

print('Optimized structure')
print(opt_molecule)

# frequencies calculation
qc_input = QchemInput(opt_molecule,
                      jobtype='freq',
                      exchange='hf',
                      basis='sto-3g',
                      scf_guess=electronic_structure['coefficients'])

parsed_data = get_output_from_qchem(qc_input,
                                    processors=4,
                                    parser=basic_frequencies,
                                    store_full_output=True)


# print results
print('Normal modes\n')

for i, mode in enumerate(parsed_data['modes']):
    print('mode:            {:10}'.format(i+1))
    print('frequency:       {:10.5f} {}'.format(mode['frequency'], mode['frequency_units']))
    print('force constant:  {:10.5f} {}\n'.format(mode['force_constant'], mode['force_constant_units']))

print('Thermochemistry\n')
for k, v in parsed_data['thermochemistry'].items():
    print('{:20} {}'.format(k, v))


# Thermodynamics
def get_thermodynamics(T):
    h_bar = 6.62607015e-34  # Plank constant J * s
    kb = 1.38064852e-23  # Boltzmann constant J / K
    cm_to_hz = 29979245800  # Hz/cm-1
    na = 6.02214076e23  # Avogadro constant
    joule_to_cal = 0.239005

    free_energy = 0
    entropy = 0
    total_energy = 0
    for mode in parsed_data['modes']:
        freq_hz = mode['frequency'] * cm_to_hz
        fact = np.exp(-h_bar * freq_hz / (kb * T))
        free_energy += freq_hz * h_bar / 2 + kb * T * np.log(1.0 - fact)
        entropy += -kb * np.log(1.0 - fact) + h_bar * freq_hz / T * fact/(1-fact)
        total_energy += freq_hz * h_bar/2 + h_bar*freq_hz * fact/(1-fact)

    free_energy *= na * joule_to_cal  # cal/mol
    entropy *= na * joule_to_cal  # cal/mol.K
    total_energy *= na * joule_to_cal  # cal/mol

    return free_energy, entropy, total_energy


T = 298.15  # Kelvin
free_energy, entropy, total_energy = get_thermodynamics(T)

# U = F + TS
print('Thermodynamics at {} K'.format(T))
print('Vibrational free energy (F): {:.4f} cal/mol'.format(free_energy))
print('Vibrational entropy (S): {:.4f} cal/mol.K'.format(entropy))
print('Total vibrational energy (U): {:.4f} cal/mol'.format(total_energy))

free_energy = []
entropy = []
total_energy = []
temp_range = range(1, 300)
for T in temp_range:
    fe, en, te = get_thermodynamics(T)
    free_energy.append(fe)
    entropy.append(en)
    total_energy.append(te)

plt.figure(1)
plt.plot(entropy, label='Entropy')
plt.xlabel('Temperature [K]')
plt.ylabel('cal/mol.K')
plt.legend()

plt.figure(2)
plt.plot(temp_range, free_energy, label='Free energy')
plt.plot(temp_range, total_energy, label='Total energy')
plt.xlabel('Temperature [K]')
plt.ylabel('cal/mol')
plt.legend()

plt.show()



from pyqchem import get_output_from_qchem, Structure, QchemInput
from pyqchem.parsers.parser_optimization import basic_optimization
from pyqchem.parsers.parser_rasci import parser_rasci
from pyqchem.file_io import build_fchk
from pyqchem.symmetry import get_orbital_classification
from pyqchem.qchem_core import redefine_calculation_data_filename
from pyqchem.utils import get_inertia
from pyqchem.units import DEBYE_TO_AU, EV_TO_NM
from pyqchem.tools import print_excited_states
from pyqchem.symmetry import get_state_symmetry

import numpy as np
import matplotlib.pyplot as plt


def draw_molecule(molecule):
    import mogli

    mol = mogli.Molecule(positions=molecule.get_coordinates(),
                         atomic_numbers=molecule.get_atomic_numbers())

    mogli.show(mol, bonds_param=1.5, title='Molecule')

redefine_calculation_data_filename('naphtalene.pkl')


# define molecule
coordinates = [[ 2.4610326539,  0.7054950347, -0.0070507104],
                 [ 1.2697800226,  1.4213478618,  0.0045894884],
                 [ 0.0071248839,  0.7134976955,  0.0071917580],
                 [-1.2465927908,  1.4207541565,  0.0039025332],
                 [ 2.4498274919, -0.7358510124,  0.0046346543],
                 [ 3.2528295760,  1.2280710625, -0.0312673955],
                 [ 1.3575083440,  2.3667492466,  0.0220260183],
                 [-1.2932627225,  2.3688000888, -0.0152164523],
                 [ 3.2670227933, -1.2176289251,  0.0251089819],
                 [-2.4610326539, -0.7054950347,  0.0070507104],
                 [-1.2697800226, -1.4213478618, -0.0045894884],
                 [-0.0071248839, -0.7134976955, -0.0071917580],
                 [ 1.2465927908, -1.4207541565, -0.0039025332],
                 [-2.4498274919,  0.7358510124, -0.0046346543],
                 [-3.2528295760, -1.2280710625,  0.0312673955],
                 [-1.3575083440, -2.3667492466, -0.0220260183],
                 [ 1.2932627225, -2.3688000888,  0.0152164523],
                 [-3.2670227933,  1.2176289251, -0.0251089819]]

symbols = ['C', 'C', 'C', 'C', 'C', 'H', 'H', 'H', 'H',
           'C', 'C', 'C', 'C', 'C', 'H', 'H', 'H', 'H']

molecule = Structure(coordinates=coordinates,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

#print(molecule.get_xyz())
# draw_molecule(molecule)

print('Initial structure')
print(molecule)


qc_input_hf = QchemInput(molecule,
                         unrestricted=True,
                         jobtype='sp',
                         exchange='hf',
                         basis='sto-3g'
                         )

#print(qc_input.get_txt())
parsed_data, electronic_structure = get_output_from_qchem(qc_input_hf,
                                                          processors=4,
                                                          force_recalculation=False,
                                                          # parser=rasci,
                                                          read_fchk=True,
                                                          store_full_output=True,
                                                          )

# create qchem input
qc_input = QchemInput(molecule,
                      jobtype='sp',
                      exchange='hf',
                      unrestricted=False,
                      correlation='rasci',
                      basis='sto-3g',
                      scf_guess=electronic_structure['coefficients'],
                      ras_act=2,
                      ras_elec=2,
                      # ras_occ=31,
                      ras_spin_mult=0,
                      ras_roots=15,
                      ras_do_hole=True,
                      ras_do_part=True,
                      # ras_srdft_cor='srpbe',
                      # ras_srdft_exc='srpbe',
                      # ras_omega=300,
                      # skip_scfman=True,
                      # mom_start=True,
                      set_iter=60)


#print(qc_input.get_txt())
parsed_data, electronic_structure = get_output_from_qchem(qc_input,
                                                          processors=4,
                                                          force_recalculation=False,
                                                          parser=parser_rasci,
                                                          read_fchk=True,
                                                          store_full_output=True,
                                                          )

# open('reference.fchk', 'w').write(build_fchk(electronic_structure))

# get orbital type
orbital_types = get_orbital_classification(electronic_structure,
                                           center=[0.0, 0.0, 0.0],
                                           orientation=[0.0, 0.0, 1.0])

# print results
print('  {:5} {:5} {:5}'.format('num', 'type', 'overlap'))
for i, ot in enumerate(orbital_types):
    print('{:5}:  {:5} {:5.3f}'.format(i + 1, ot[0], ot[1]))

# print(molecule.alpha_electrons, molecule.beta_electrons)

print('\nAdiabatic states\n--------------------')
print_excited_states(parsed_data['excited_states'], include_conf_rasci=True)

inertia_moments, inertia_axis = get_inertia(electronic_structure['structure'])

#print(im)
#print(ivec[0])
#print(ivec[1])
#print(ivec[2])

print('\nInertia\n----------')
print(inertia_moments)
print(np.array(inertia_axis))

print(electronic_structure['structure'])
symmetry_measures = get_state_symmetry(electronic_structure,
                                       parsed_data['excited_states'],
                                       center=[0, 0, 0],
                                       orientation=inertia_axis[0],
                                       orientation2=inertia_axis[1],
                                       group='D2h',
                                       extra_print=False
                                       )

energies = [state['excitation_energy'] for state in parsed_data['excited_states']]

print('\nSymmetry of RASCI excited states\n--------------------------------')
for energy, state in zip(energies, symmetry_measures.items()):
    print('{:8} '.format(state[0]) + ' {:5} {:5.3f}'.format(*state[1]) + '  {:5.3f} eV'.format(energy))




from pyqchem.structure import Structure
from pyqchem.qchem_core import get_output_from_qchem, create_qchem_input
from pyqchem.parsers.basic import basic_parser_qchem
import numpy as np


parameters = {'jobtype': 'sp',
              'exchange': 'hf',
              'basis': '6-31G',
              # cis
              'cis_n_roots': 20,
              'cis_convergence': 8,
              'cis_singlets': True,
              'cis_triplets': False,
              'cis_ampl_anal': True,
              # other
              'RPA': False,
              'gui': 0}

distance = 4.0

slide_y = 0
slide_z = 0

range_z = [0, 0.25, 0.5, 0.75, 1, 1.25, 1.5, 1.75, 2, 2.25, 2.5, 2.75, 3, 3.25, 3.5, 3.75, 4]
range_y = [0]
for slide_y in range_y:
    for slide_z in range_z:
        print('y:{}  z:{}'.format(slide_y, slide_z))
        coordinates = [[0.0000000, 0.0000000, 0.6660120],
                       [0.0000000, 0.0000000, -0.6660120],
                       [0.0000000, 0.9228100, 1.2279200],
                       [0.0000000, -0.9228100, 1.2279200],
                       [0.0000000, -0.9228100, -1.2279200],
                       [0.0000000, 0.9228100, -1.2279200],
                       [distance, 0.0000000, 0.6660120],
                       [distance, 0.0000000, -0.6660120],
                       [distance, 0.9228100, 1.2279200],
                       [distance, -0.9228100, 1.2279200],
                       [distance, -0.9228100, -1.2279200],
                       [distance, 0.9228100, -1.2279200]]

        coordinates = np.array(coordinates)

        coordinates[6:, 1] = coordinates[6:, 1] + slide_y
        coordinates[6:, 2] = coordinates[6:, 2] + slide_z

        molecule = Structure(coordinates=coordinates,
                             symbols=['C', 'C', 'H', 'H', 'H', 'H', 'C', 'C', 'H', 'H', 'H', 'H'],
                             charge=0)

        txt_input = create_qchem_input(molecule, **parameters)
        # print(txt_input)

        data = get_output_from_qchem(txt_input, processors=4, force_recalculation=False, parser=basic_parser_qchem)

        # if closed shell
        ocup_orb = (np.sum(molecule.get_atomic_numbers()) - molecule.charge) // 2
        n_states = 2

        # get list of interesting states
        interesting_transitions = []
        for i, state in enumerate(data['excited states cis']):
            for transition in state['transitions']:
                if transition['origin'] > ocup_orb - n_states and transition['target'] <= n_states:
                    interesting_transitions.append(
                        [i + 1, transition['origin'], transition['target'], transition['amplitude'] ** 2])

        interesting_transitions = np.array(interesting_transitions)
        list_states = interesting_transitions[np.array(interesting_transitions)[:, 3].argsort()]
        list_states = np.array(list_states[:, 0][::-1], dtype=int)
        indexes = np.unique(list_states, return_index=True)[1]
        list_states = np.sort([list_states[index] for index in sorted(indexes)][:n_states * 2])
        # print(list_states)
        trans_moments = []
        for state in list_states:
            trans_moments.append(data['excited states cis'][state]['transition moment'])
        print(trans_moments)

        parameters.update({'loc_cis_ov_separate': False,
                           'er_cis_numstate': n_states * 2,
                           'cis_diabath_decompose': True,
                           'localized_diabatization': list_states})
        txt_input = create_qchem_input(molecule, **parameters)
        # print(txt_input)
        data = get_output_from_qchem(txt_input, force_recalculation=True, parser=None)
        with open('qchemout_{}_{}.out'.format(slide_y, slide_z), 'w') as f:
            f.write(data)



import pickle
import numpy as np
import matplotlib.pyplot as plt
import argparse
from pyqchem.order_states import get_order_states_list, correct_order_list
import matplotlib

matplotlib.rcParams.update({'font.size': 13})

# Argument parser
parser = argparse.ArgumentParser(description='Plot 3D data')
parser.add_argument('filename', metavar='filename', type=str,
                    help='filename for input')

parser.add_argument('--output_folder', metavar='distance', type=str, default='1d_plot/',
                    help='folder to store PDF plots')

parser.add_argument('--show_plots', action='store_true',
                   help='show plots while running')


args = parser.parse_args()


def multiplot(data , labels, x_range, title=None, factor=None, range_y=(-1.0, 1.0),
              ylabel='Energy [eV]', show_plots=False, colors=None, baseline=None, style=None):

    if style is None:
        style = [None for i in range(len(data))]

    if factor is not None:
        for i, dat in enumerate(data):
            data[i] = factor * np.array(dat)

    f = plt.figure(1)

    plt.title(title)
    for i, dat in enumerate(data):
        if colors is not None:
            plt.plot(x_range, dat, label=labels[i], color=colors[i], linestyle=style[i])
        else:
            plt.plot(x_range, dat, label=labels[i], linestyle=style[i])

    plt.xlim(3.5, 6.0)
    if range_y is not None:
        plt.ylim(*range_y)
    plt.legend()
    #leg = plt.legend(loc='center right')
    #leg.get_frame().set_linewidth(0.0)

    plt.xlabel('Z [Å]')
    plt.ylabel(ylabel)

    if baseline is not None:
        plt.axhline(baseline, color='k', linestyle=':')

    if show_plots:
        plt.show()
    else:
        plt.close()

    return f


def multiplot_2axis(data , labels, data2, labels2, x_range, title=None, factor=None, range_y=(-1.0, 1.0),
                    ylabel='Energy [eV]', show_plots=False, colors=None, style1=None, style2=None,
                    colors2=None, ylabel2=None, range_y2=(-1, 1), baseline=None):

    if style1 is None:
        style1 = [None for i in range(len(data))]

    if style2 is None:
        style2 = [None for i in range(len(data2))]

    if factor is not None:
        for i, dat in enumerate(data):
            data[i] = factor[0] * np.array(dat)
        for i, dat in enumerate(data2):
            data2[i] = factor[1] * np.array(dat)

    f, ax1 = plt.subplots()

    plt.title(title)
    plt.xlim(3.5, 6.0)

    lns1 = []
    for i, dat in enumerate(data):
        if colors is not None:
            lns1.append(ax1.plot(x_range, dat, label=labels[i], color=colors[i], linestyle=style1[i]))
        else:
            lns1.append(ax1.plot(x_range, dat, label=labels[i], linestyle=style1[i]))

    if range_y is not None:
        ax1.set_ylim(*range_y)
    ax1.set_ylabel(ylabel)

    ax2 = ax1.twinx()

    lns2 = []
    for i, dat in enumerate(data2):
        if colors2 is not None:
            lns2.append(ax2.plot(x_range, dat, label=labels2[i], color=colors2[i]))
        else:
            lns2.append(ax2.plot(x_range, dat, label=labels2[i]))

    if range_y2 is not None:
        ax2.set_ylim(*range_y2)

    ax2.set_ylabel(ylabel2)

    lns = [j for i in lns1 for j in i] + [j for i in lns2 for j in i]
    labs = [l.get_label() for l in lns]
    plt.legend(lns, labs, loc=0)

    if baseline is not None:
        ax1.axhline(baseline, color='k', linestyle=':')

    ax1.set_xlabel('Z [Å]')

    if show_plots:
        plt.show()
    else:
        plt.close()

    return f


#############################
folder = args.output_folder
#############################


with open(args.filename, 'rb') as input:
    calculation_data = pickle.load(input)
    print('Loaded data from calculation_data.pkl')

total_data = []
d_coordinate = []
for distance in calculation_data['distance']:
        print('{}'.format(distance))
        if '{}'.format(distance) in calculation_data:
            data = calculation_data['{}'.format(distance)]
            total_data.append(data)
            d_coordinate.append(distance)

states_orders = [get_order_states_list(data['states_info'], eps_energy=0) for data in total_data]

########################### W_DC ###########################

data_1 = [data['diabatic_contributions']['W_DC'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['W_DC'][1] for data in total_data]
data_3 = [data['diabatic_contributions']['W_DC'][2] for data in total_data]
data_4 = [data['diabatic_contributions']['W_DC'][3] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

f = multiplot([data_1, data_2], ['$W_{DC}^{(1)}$', '$W_{DC}^{(2)}$'], d_coordinate, title='W_DC',
              show_plots=args.show_plots, range_y=[None, None])
f.savefig(folder + "W_DC.pdf", bbox_inches='tight')

wdc1 = np.array(data_1)
wdc2 = np.array(data_2)
wdc3 = np.array(data_3)
wdc4 = np.array(data_4)


########################### W_CT ###########################

data_1 = [data['diabatic_contributions']['W_CT'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['W_CT'][1] for data in total_data]
data_3 = [data['diabatic_contributions']['W_CT'][2] for data in total_data]
data_4 = [data['diabatic_contributions']['W_CT'][3] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

f = multiplot([data_1, data_2], ['$W_{CT}^{(1)}$', '$W_{CT}^{(2)}$'], d_coordinate, title='W_CT',
              show_plots=args.show_plots, range_y=[None, None])
f.savefig(folder + "W_CT.pdf", bbox_inches='tight')

wct1 = np.array(data_1)
wct2 = np.array(data_2)
wct3 = np.array(data_3)
wct4 = np.array(data_4)

########################### W_e ############################

data_1 = [data['diabatic_contributions']['W_e'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['W_e'][1] for data in total_data]
data_3 = [data['diabatic_contributions']['W_e'][2] for data in total_data]
data_4 = [data['diabatic_contributions']['W_e'][3] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

f = multiplot([data_1, data_2], ['$W_e^{(1)}$', '$W_e^{(2)}$'], d_coordinate, title='W_e',
              show_plots=args.show_plots, range_y=[None, None])
f.savefig(folder + "W_e.pdf", bbox_inches='tight')

we1 = np.array(data_1)
we2 = np.array(data_2)
we3 = np.array(data_3)
we4 = np.array(data_4)

########################### W_h ############################

data_1 = [data['diabatic_contributions']['W_h'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['W_h'][1] for data in total_data]
data_3 = [data['diabatic_contributions']['W_h'][2] for data in total_data]
data_4 = [data['diabatic_contributions']['W_h'][3] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

f = multiplot([data_1, data_2], ['$W_h^{(1)}$', '$W_h^{(2)}$'], d_coordinate, title='W_h',
              show_plots=args.show_plots, range_y=[None, None])
f.savefig(folder + "W_h.pdf", bbox_inches='tight')

wh1 = np.array(data_1)
wh2 = np.array(data_2)
wh3 = np.array(data_3)
wh4 = np.array(data_4)

###################### W per state ###########################

f = multiplot([wh1, we1, wct1, wdc1], ['$W_h$', '$W_e$', '$W_{CT}$', '$W_{DC}$'], d_coordinate,
              title='State 1', show_plots=args.show_plots, range_y=[None, None])
f.savefig(folder + "W_1.pdf", bbox_inches='tight')

f = multiplot([wh2, we2, wct2, wdc2], ['$W_h$', '$W_e$', '$W_{CT}$', '$W_{DC}$'], d_coordinate,
              title='State 2', show_plots=args.show_plots, range_y=[None, None])
f.savefig(folder + "W_2.pdf", bbox_inches='tight')


###################################### lambda ###########################

data_1 = [data['asymmetries']['lambda2'][0] for data in total_data]
data_2 = [data['asymmetries']['lambda2'][1] for data in total_data]
data_3 = [data['asymmetries']['lambda2'][2] for data in total_data]
data_4 = [data['asymmetries']['lambda2'][3] for data in total_data]

#data_1 = []
#for i, e in enumerate(np.array([data['lambda'] for data in total_data]).T[0]):
#    data_1.append(e)

#data_2 = []
#for i, e in enumerate(np.array([data['lambda'] for data in total_data]).T[1]):
#    data_2.append(e)

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

f = multiplot([data_1, data_2], ['$\lambda^{2(1)}$', '$\lambda^{2(2)}$'], d_coordinate, range_y=[None, None],
              ylabel='', title='lambda^2', show_plots=args.show_plots)
f.savefig(folder + "lambda2.pdf", bbox_inches='tight')

l1 = np.array(data_1)
l2 = np.array(data_2)
l3 = np.array(data_3)
l4 = np.array(data_4)

##########################  OMEGA  #######################

data_1 = [data['diabatic_contributions']['Omega_e'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['Omega_e'][1] for data in total_data]

data_3 = [data['diabatic_contributions']['Omega_h'][0] for data in total_data]
data_4 = [data['diabatic_contributions']['Omega_h'][1] for data in total_data]

#data_1 = 2 * l1 * np.sqrt(1 - l1**2) * we1
#data_2 = 2 * l2 * np.sqrt(1 - l2**2) * we2

#data_3 = 2 * l1 * np.sqrt(1 - l1**2) * wh1
#data_4 = 2 * l2 * np.sqrt(1 - l2**2) * wh2

f = multiplot([data_1, data_2, data_3, data_4], ['$\Omega_e^{(1)}$', '$\Omega_e^{(2)}$', '$\Omega_h^{(1)}$', '$\Omega_h^{(2)}$'],
              d_coordinate, ylabel='', title='Omega', show_plots=args.show_plots, range_y=[None, None])
f.savefig(folder + "Omega.pdf", bbox_inches='tight')

oe_1 = np.array(data_1)
oe_2 = np.array(data_2)
oh_1 = np.array(data_3)
oh_2 = np.array(data_4)


########################## delta ###########################

e_le = np.array([data['diabatic_energies']['E_LE'] for data in total_data])
e_ct = np.array([data['diabatic_energies']['E_CT'] for data in total_data])
de_le = np.array([data['diabatic_energies']['dE_LE'] for data in total_data])
de_ct = np.array([data['diabatic_energies']['dE_CT'] for data in total_data])

d2le1 = np.array([data['asymmetries']['delta2_le'][0] for data in total_data])
d2le2 = np.array([data['asymmetries']['delta2_le'][1] for data in total_data])

d2ct1 = np.array([data['asymmetries']['delta2_ct'][0] for data in total_data])
d2ct2 = np.array([data['asymmetries']['delta2_ct'][1] for data in total_data])


data_1 = d2le1 * de_le + d2ct1 * de_ct + l1 * (e_ct - e_le)
data_2 = d2le2 * de_le + d2ct2 * de_ct + l2 * (e_ct - e_le)


f = multiplot([data_1, data_2], ['$\Delta_1$', '$\Delta_2$'], d_coordinate, show_plots=args.show_plots, #range_y=[None, None],
              title='$\Delta$')
f.savefig(folder + "delta.pdf", bbox_inches='tight')

f = multiplot([e_le, e_ct, de_le, de_ct], ['$E_{LE}$', '$E_{CT}$', '$\Delta E_{LE}$', '$\Delta E_{CT}$'],
              d_coordinate, show_plots=args.show_plots, range_y=[None, None], title='term1 pieces')
f.savefig(folder + "diabatic_1.pdf", bbox_inches='tight')

f = multiplot([d2le1, d2ct1, d2le2, d2ct2], ['$\delta^{2(1)}_{LE}$', '$\delta^{2(1)}_{CT}$',
                                             '$\delta^{2(2)}_{LE}$', '$\delta^{2(2)}_{CT}$'],
              d_coordinate, show_plots=args.show_plots, title='term1 pieces')
f.savefig(folder + "diabatic_2.pdf", bbox_inches='tight')


delta_1 = np.array(data_1)
delta_2 = np.array(data_2)

#######################  J Coloumb ######################

mu = 1.7528
angs2au = 1.889725989
har2ev = 27.2113962
j_coul = np.array([(mu**2)/(z * angs2au)**3 * har2ev for z in d_coordinate])

#######################  Omega DC  #####################

data_1 = np.sqrt((1-l1)**2 - d2le1**2)*wdc1
data_2 = np.sqrt((1-l2)**2 - d2le2**2)*wdc2

add_coul = True
if add_coul:
    f = multiplot([data_1, data_2, j_coul, -j_coul],
                  ['$\Omega_{DC}^{(1)}$', '$\Omega_{DC}^{(2)}$', '$J_{Coul}$', '$J_{Coul}$'],
                  d_coordinate, title='$\Omega_{DC}$', colors=['#1f77b4', '#ff7f0e', 'grey', 'grey'],
                  style=['-', '-', '--', '--'])
else:
    f = multiplot([data_1, data_2],
                  ['$\Omega_{DC}^{(1)}$', '$\Omega_{DC}^{(2)}$'],
                  d_coordinate, title='$\Omega_{DC}$')

f.savefig(folder + "omega_DC.pdf", bbox_inches='tight', show_plots=args.show_plots, range_y=[None, None])

o_dc1 = data_1
o_dc2 = data_2

#######################  Omega CT  #####################

data_1 = np.sqrt(l1**2 - d2ct1**2)*wct1
data_2 = np.sqrt(l1**2 - d2ct2**2)*wct2

f = multiplot([data_1, data_2], ['$\Omega_{CT}^{(1)}$', '$\Omega_{CT}^{(2)}$'], d_coordinate, title='$\Omega_{CT}$')
f.savefig(folder + "omega_CT.pdf", bbox_inches='tight', show_plots=args.show_plots, range_y=[None, None])

o_ct1 = data_1
o_ct2 = data_2

###################### Omega h ###########################

o_h1 = np.array([data['diabatic_contributions']['Omega_h'][0] for data in total_data])
o_h2 = np.array([data['diabatic_contributions']['Omega_h'][1] for data in total_data])

f = multiplot([o_h1, o_h2], ['$\Omega_{h}^{(1)}$', '$\Omega_{h}^{(2)}$'], d_coordinate, title='$\Omega_{h}$')
f.savefig(folder + "omega_h.pdf", bbox_inches='tight', show_plots=args.show_plots, range_y=[None, None])

###################### Omega e ###########################

o_e1 = np.array([data['diabatic_contributions']['Omega_e'][0] for data in total_data])
o_e2 = np.array([data['diabatic_contributions']['Omega_e'][1] for data in total_data])

f = multiplot([o_e1, o_e2], ['$\Omega_{e}^{(1)}$', '$\Omega_{e}^{(2)}$'], d_coordinate, title='$\Omega_{e}$')
f.savefig(folder + "omega_e.pdf", bbox_inches='tight', show_plots=args.show_plots, range_y=[None, None])

#######################  diabatic_energies  ######################

data_1 = [np.average(data['diabatic_energies']['E_LE']) for data in total_data]
data_2 = [np.average(data['diabatic_energies']['E_CT']) for data in total_data]

e_le = np.array(data_1)
e_ct = np.array(data_2)

f = multiplot([data_1, data_2], ['$E_{LE}$', '$E_{CT}$'], d_coordinate, #title='Diabatic energies',
              range_y=[6, 13], show_plots=args.show_plots)
f.savefig(folder + "figure2b.pdf", bbox_inches='tight')


###################### Diabatic energies ##################

data_1 = [data['diabatic_energies']['V_DC'] for data in total_data]
data_2 = [data['diabatic_energies']['V_CT'] for data in total_data]
data_3 = [data['diabatic_energies']['V_e'][0] for data in total_data]
data_4 = [data['diabatic_energies']['V_h'][0] for data in total_data]
data_5 = [data['diabatic_energies']['V_e'][1] for data in total_data]
data_6 = [data['diabatic_energies']['V_h'][1] for data in total_data]

f = multiplot([data_1, data_2,  data_3,  data_4,  data_5, data_6],
              ['$V_{DC}$', "$V_{CT}$", "$V_e$", "$V_h$", "$V_e'$", "$V_h'$"],
              d_coordinate, #title='Diabatic energies',
              show_plots=args.show_plots)

f.savefig(folder + "diabatic_energies.pdf", bbox_inches='tight')

###################### Adiabatic energies ##################

data_1 = [data['adiabatic_energies']['E_1'] for data in total_data]
data_2 = [data['adiabatic_energies']['E_2'] for data in total_data]
data_3 = [data['adiabatic_energies']['E_3'] for data in total_data]
data_4 = [data['adiabatic_energies']['E_4'] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

f = multiplot([data_1, data_2,  data_3,  data_4,],
              #['$1 - {}^1A_g$', '$1 - {}^1B_{3u}$', '$2- {}^1B_{3u}$', '$2 - {}^1A_g$'],
              ['${}^1A_g$', '${}^1B_{3u}$', None, None],
              #['$E_1$', '$E_2$', '$E_3$' , '$E_4$'],
              d_coordinate, # title='Adiabatic energies',
              range_y=[6, 13], show_plots=args.show_plots,
              colors=['black', 'red', 'black', 'red'],
              )

f.savefig(folder + "figure2a.pdf", bbox_inches='tight')

e1 = np.array(data_1)
e2 = np.array(data_2)
e3 = np.array(data_3)
e4 = np.array(data_4)


f = multiplot([data_1, data_2,  data_3,  data_4, e_le, e_ct],
              #['$1 - {}^1A_g$', '$1 - {}^1B_{3u}$', '$2- {}^1B_{3u}$', '$2 - {}^1A_g$'],
              ['${}^1A_g$', '${}^1B_{3u}$', None, None, '$E_{LE}$', '$E_{CT}$'],
              #['$E_1$', '$E_2$', '$E_3$' , '$E_4$'],
              d_coordinate, # title='Adiabatic energies',
              range_y=[6, 13], show_plots=args.show_plots,
              colors=['black', 'red', 'black', 'red', None, None],
              )

f.savefig(folder + "figure2_mix.pdf", bbox_inches='tight')



#######################  Figure 3  #####################

o_sx_1 = o_e1 + o_h1
o_sx_2 = o_e2 + o_h2

di_1 = l1*(e_ct - e_le) + d2le1*de_le + d2ct1*de_ct
di_2 = l2*(e_ct - e_le) + d2le2*de_le + d2ct2*de_ct


f = multiplot_2axis([o_dc1, o_sx_1, o_ct1, di_1, -j_coul], ['$\Omega^{(1)}_{DC}$', '$\Omega_{SX}^{(1)}$',
                                                            '$\Omega_{CT}^{(1)}$', '$\Delta_1$', '$J_{Coul}$'],
                    [l1], ['$\lambda_1^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    colors=[None, None, None, None, '#1f77b4'], style1=['-', '-', '-', '-', '--'],
                    range_y2=[0, 0.2], title='State 1', colors2=['grey'], baseline=0)
f.savefig(folder + "figure3a.pdf", bbox_inches='tight')


f = multiplot_2axis([o_dc2, o_sx_2, o_ct2, di_2, j_coul], ['$\Omega^{(2)}_{DC}$', '$\Omega_{SX}^{(2)}$',
                                                           '$\Omega_{CT}^{(2)}$', '$\Delta_2$', '$J_{Coul}$'],
                    [l2], ['$\lambda_2^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    colors=[None, None, None, None, '#1f77b4'], style1=['-', '-', '-', '-','--'],
                    range_y2=[0, 0.2], title='State 2', colors2=['grey'], baseline=0)

f.savefig(folder + "figure3b.pdf", bbox_inches='tight')


f = multiplot_2axis([oe_1, oh_1, o_sx_1], ['$\Omega_e^{(1)}$', '$\Omega_h^{(1)}$', '$\Omega_{SX}^{(1)}$'],
                    [l1], ['$\lambda_1^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    range_y2=[0.0, 0.2], title='State 1', colors2=['grey'], baseline=0)

f.savefig(folder + "figure3c.pdf", bbox_inches='tight')

f = multiplot_2axis([oe_2, oh_2, o_sx_1], ['$\Omega_e^{(2)}$', '$\Omega_h^{(2)}$', '$\Omega_{SX}^{(2)}$'],
                    [l2], ['$\lambda_2^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    range_y2=[0.0, 0.2], title='State 2', colors2=['grey'], baseline=0)

f.savefig(folder + "figure3d.pdf", bbox_inches='tight')



# Figure omega

data_1 = [data['diabatic_contributions']['Omega_One_dc'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['Omega_One_dc'][0] for data in total_data]
data_3 = [data['diabatic_contributions']['Omega_One_dc'][0] for data in total_data]
data_4 = [data['diabatic_contributions']['Omega_One_dc'][0] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

o_one_dc = data_1

data_1 = [data['diabatic_contributions']['Omega_J_dc'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['Omega_J_dc'][0] for data in total_data]
data_3 = [data['diabatic_contributions']['Omega_J_dc'][0] for data in total_data]
data_4 = [data['diabatic_contributions']['Omega_J_dc'][0] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

o_j_dc = data_1

data_1 = [data['diabatic_contributions']['Omega_K_dc'][0] for data in total_data]
data_2 = [data['diabatic_contributions']['Omega_K_dc'][0] for data in total_data]
data_3 = [data['diabatic_contributions']['Omega_K_dc'][0] for data in total_data]
data_4 = [data['diabatic_contributions']['Omega_K_dc'][0] for data in total_data]

data_1, data_2, data_3, data_4 = correct_order_list([data_1, data_2, data_3, data_4], states_orders)

o_k_dc = data_1

f = multiplot_2axis([o_dc1, o_one_dc, o_j_dc,  o_k_dc, -j_coul],
                    ['$\Omega_{DC}^{(1)}$', '$\Omega_{DC,1}^{(1)}$', '$\Omega_{DC,J}^{(1)}$', '$\Omega_{DC,K}^{(1)}$', '$J_{Coul}$'],
                    [l1], ['$\lambda_1^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    colors=[None, None, None, None, '#2ca02c'], style1=['-', '-', '-', '-', '--'],
                    range_y2=[0, 0.2], colors2=['grey'], baseline=0)

f.savefig(folder + "omega_other.pdf", bbox_inches='tight')

exit()

f = multiplot_2axis([oe_1, oh_1], ['$\Omega_e^{(1)}$', '$\Omega_h^{(1)}$'],
                    [np.square(l1)], ['$\lambda_1^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    style1=['-', '-.'], colors=['black', 'black'],
                    range_y2=[-0.3, 0.3], title='State 1', colors2=['grey'], baseline=0)

f.savefig(folder + "figure3c_x.pdf", bbox_inches='tight')

f = multiplot_2axis([oe_2, oh_2], ['$\Omega_e^{(2)}$', '$\Omega_h^{(2)}$'],
                    [np.square(l2)], ['$\lambda_2^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    style1=['-', '-.'], colors=['red', 'red'],
                    range_y2=[-0.3, 0.3], title='State 2', colors2=['grey'], baseline=0)

f.savefig(folder + "figure2d_x.pdf", bbox_inches='tight')



exit()

#######################  adiabatic_energies (calculated) ######################

e1c = e_le + wdc1 + e_11 + e_21
e2c = e_le + wdc2 + e_12 + e_22

f = multiplot([e1-e1c, e2-e2c],
              ['diff 1', 'diff 2'],
              d_coordinate, title='Adiabatic energies difference\n (original-calculated)',
              range_y=None, show_plots=args.show_plots)

f.savefig(folder + "test.pdf", bbox_inches='tight')

#######################  Figure 3  #####################

f = multiplot_2axis([wdc1, e_11, e_21, -j_coul], ['$W^{(1)}_{DC}$', '$\Omega^{(1)}$', '$E(\lambda^2_1)$', '$J_{Coul}$'],
                    [np.square(l1)], ['$\lambda_1^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    colors=[None, None, None, '#1f77b4'], style1=['-', '-', '-', '--'],
                    range_y2=[-0.3, 0.3], title='State 1', colors2=['grey'], baseline=0)
f.savefig(folder + "figure2a.pdf", bbox_inches='tight')

f = multiplot_2axis([wdc2, e_12, e_22, j_coul], ['$W^{(2)}_{DC}$', '$\Omega^{(2)}$', '$E(\lambda^2_2)$', '$J_{Coul}$'],
                    [np.square(l2)], ['$\lambda_2^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    colors=[None, None, None, '#1f77b4'], style1=['-', '-', '-', '--'],
                    range_y2=[-0.3, 0.3], title='State 2', colors2=['grey'], baseline=0)

f.savefig(folder + "figure2b.pdf", bbox_inches='tight')

f = multiplot_2axis([oe_1, oh_1, e_11], ['$\Omega_e^{(1)}$', '$\Omega_h^{(1)}$', '$\Omega^{(1)}$'],
                    [np.square(l1)], ['$\lambda_1^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    range_y2=[-0.3, 0.3], title='State 1', colors2=['grey'], baseline=0)

f.savefig(folder + "figure2c.pdf", bbox_inches='tight')

f = multiplot_2axis([oe_2, oh_2, e_12], ['$\Omega_e^{(2)}$', '$\Omega_h^{(2)}$', '$\Omega^{(2)}$'],
                    [np.square(l2)], ['$\lambda_2^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    range_y2=[-0.3, 0.3], title='State 2', colors2=['grey'], baseline=0)

f.savefig(folder + "figure2d.pdf", bbox_inches='tight')


f = multiplot_2axis([oe_1, oh_1], ['$\Omega_e^{(1)}$', '$\Omega_h^{(1)}$'],
                    [np.square(l1)], ['$\lambda_1^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    style1=['-', '-.'], colors=['black', 'black'],
                    range_y2=[-0.3, 0.3], title='State 1', colors2=['grey'], baseline=0)

f.savefig(folder + "figure2c_x.pdf", bbox_inches='tight')

f = multiplot_2axis([oe_2, oh_2], ['$\Omega_e^{(2)}$', '$\Omega_h^{(2)}$'],
                    [np.square(l2)], ['$\lambda_2^2$'], d_coordinate, ylabel2='$\lambda^2$',
                    style1=['-', '-.'], colors=['red', 'red'],
                    range_y2=[-0.3, 0.3], title='State 2', colors2=['grey'], baseline=0)

f.savefig(folder + "figure2d_x.pdf", bbox_inches='tight')




from pyqchem import get_output_from_qchem, Structure, QchemInput
from pyqchem.parsers.parser_optimization import basic_optimization
import matplotlib.pyplot as plt
from pyqchem.errors import OutputError

# define molecule
coordinates = [[0.0,  1.0,  0.0],
               [0.0,  0.0,  1.0],
               [0.0,  0.0, -1.0]]

symbols = ['O', 'H', 'H']

molecule = Structure(coordinates=coordinates,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

print('Initial structure')
print(molecule)

# Optimization constrains definition
constrains = {'stre': [{'atoms': [1, 2], 'value': 0.9},
                       {'atoms': [1, 3], 'value': 0.9}],
              'bend': [{'atoms': [2, 1, 3], 'value': 110.0}]}

qc_input = QchemInput(molecule,
                      jobtype='opt',
                      exchange='hf',
                      basis='sto-3g',
                      geom_opt_tol_gradient=300,
                      geom_opt_tol_energy=100,
                      geom_opt_tol_displacement=1200,
                      geom_opt_max_cycles=50,  # reduce this number to test not convergence case
                      geom_opt_constrains=constrains  # comment/uncomment this line to use or not constrains
                      )

try:
    parsed_data = get_output_from_qchem(qc_input,
                                        processors=4,
                                        parser=basic_optimization,
                                        force_recalculation=False)

    opt_molecule = parsed_data['optimized_molecule']

    print('Optimized structure')
    print(opt_molecule)
    print('Final energy:', parsed_data['energy'])

except OutputError as e:
    print(e.error_lines)
    parsed_data = basic_optimization(e.full_output)
    print('Last structure')
    print(parsed_data['optimization_steps'][-1]['molecule'])
    print('Last energy:', parsed_data['optimization_steps'][-1]['energy'])


energies = [step['energy'] for step in  parsed_data['optimization_steps']]
gradients = [step['gradient'] for step in parsed_data['optimization_steps']]
displacements = [step['displacement'] for step in parsed_data['optimization_steps']]

# Plot data
fig, axs = plt.subplots(3, sharex=True, gridspec_kw={'hspace': 0})
fig.suptitle('Optimization steps')
axs[0].set(ylabel='Energy')
axs[0].plot(energies, color='b')
axs[1].set(ylabel='Gradient')
axs[1].plot(gradients, color='r')
axs[2].set(ylabel='Displacement', xlabel='Steps')
axs[2].plot(displacements, color='g')

plt.show()